import unittest

import numpy as np
import pandas as pd
import seaborn as sns
import shutil
from pathlib import Path

from aethos.core import Data


class Test_TestBase(unittest.TestCase):
    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(str(Path.home()) + "/.aethos/reports/")

    def test_compare_dist_predict(self):

        data = np.random.randint(0, 2, size=(1000, 3))
        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        df = Data(data, target_field="col3", report_name="test")
        df.predict_data_sample()

        self.assertTrue(True)

    def test_compare_dist_ks(self):

        data = np.random.randint(0, 2, size=(1000, 3))
        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        df = Data(data, target_field="col3", report_name="test")
        df["col4"] = np.random.normal(1, 2, size=(1, 800))[0]
        df["col4"] = np.random.normal(10, 20, size=(1, 200))[0]
        df.ks_feature_distribution()

        self.assertTrue(True)



import warnings

warnings.simplefilter("ignore", FutureWarning)

import pandas as pd

# let init-time option registration happen
import aethos.config.config_init
import shap
from bokeh.io import output_notebook
from IPython import get_ipython
from aethos.config.config import (
    describe_option,
    get_option,
    options,
    reset_option,
    set_option,
)

from .core import Data
from .modelling import Model

pd.options.mode.chained_assignment = None

__all__ = ["Data", "Model"]

shell = get_ipython().__class__.__name__

if shell == "ZMQInteractiveShell":
    output_notebook()
    shap.initjs()



import os
import platform
import subprocess
from datetime import datetime
from aethos.config import cfg, DEFAULT_REPORT_DIR, DEFAULT_IMAGE_DIR
from aethos.config.config import _global_config
from aethos.util import _make_dir
from docx import Document
from docx.enum.style import WD_STYLE_TYPE
from docx.shared import Inches, Pt
import pandas as pd


class Report:
    def __init__(self, report_name):

        if not cfg["report"]["dir"]:  # pragma: no cover
            report_dir = DEFAULT_REPORT_DIR
        else:
            report_dir = cfg["report"]["dir"]

        _make_dir(report_dir)

        self.report_name = report_name
        self.docx = _global_config["word_report"]

        if os.path.exists(self.report_name):
            self.filename = self.report_name
            self.docx_filename = self.filename.replace(".txt", ".docx")

            if self.docx:  # pragma: no cover
                self.doc = Document(docx=self.docx_filename)
                style = self.doc.styles
                if not style["Indent"]:
                    style.add_style("Indent", WD_STYLE_TYPE.PARAGRAPH)
                    paragraph_format = style.paragraph_format
                    paragraph_format.left_indent = Inches(0.25)
        else:
            self.filename = "{}/{}-{}".format(
                report_dir, report_name, datetime.now().strftime("%d-%m-%Y_%I-%M-%S%p")
            )

            if self.docx:  # pragma: no cover
                self.doc = Document()
                style = self.doc.styles.add_style("Indent", WD_STYLE_TYPE.PARAGRAPH)
                paragraph_format = style.paragraph_format
                paragraph_format.left_indent = Inches(0.25)
                self.docx_filename = self.filename + ".docx"

            self.filename += ".txt"

            self.report_environtment()

        if not cfg["images"]["dir"]:
            self.image_dir = DEFAULT_IMAGE_DIR
        else:
            self.image_dir = cfg["images"]["dir"]

    def write_header(self, header: str, level=1):
        """
        Writes a header to a report file
        
        Parameters
        ----------
        header : str
            Header for a report file
            Examples: 'Cleaning', 'Feature Engineering', 'Modelling', etc.
        """

        write = True

        if os.path.exists(self.filename):
            with open(self.filename, "r") as f:
                if header in f.read():
                    write = False

        if write:
            if self.docx:  # pragma: no cover
                self.doc.add_heading(header, level=level)
                self.doc.save(self.docx_filename)

            with open(self.filename, "a+") as f:
                f.write("\n" + header + "\n\n")

    def write_contents(self, content: str):
        """
        Write report contents to report.
        
        Parameters
        ----------
        content : str
            Report Content
        """

        write = True

        if os.path.exists(self.filename):
            with open(self.filename, "r") as f:
                if content in f.read():
                    write = False

        if write:
            if self.docx:  # pragma: no cover
                self.doc.add_paragraph(content, style="Indent")
                self.doc.save(self.docx_filename)

            with open(self.filename, "a+") as f:
                f.writelines(content)

    def report_technique(self, technique: str, list_of_cols=[]):
        """
        Writes analytic technique info to the report detailing what analysis was run
        and why it was ran.
        
        Parameters
        ----------
        technique : str
            Analytic technique
            
        list_of_cols : list
            List of columns
        """

        if list(list_of_cols) != []:
            for col in list_of_cols:
                log = technique.replace("column_name_placeholder", col)

                self.write_contents(log)

        else:
            self.write_contents(technique)

    def log(self, log: str):
        """
        Logs info to the report file.
        
        Parameters
        ----------
        log : str
            Information to write to the report file
        """

        self.write_contents(log + "\n")

    def report_gridsearch(self, clf, verbose):
        """
        Logs gridsearch results.
        """

        content = f"Tuning hyper-parameters:\n\n \
            Best parameters set found on training set:\n \
                {clf.best_params_}\n\n \
            Grid scores on training set:\n\n"

        self.write_contents("\n".join(map(str.lstrip, content.split("\n"))))

        if verbose:
            print(content)

        means = clf.cv_results_["mean_test_score"]
        stds = clf.cv_results_["std_test_score"]
        for mean, std, params in zip(means, stds, clf.cv_results_["params"]):
            stats = "{:.2%} (+/-{}) for {}\n".format(float(mean), std * 2, params)
            self.write_contents(stats)
            if verbose:
                print(stats)

            self.write_contents("\n")

    def report_classification_report(self, report):
        """
        Logs classification report results.
        """

        self.write_contents("Detailed classification report:\n")
        self.write_contents("The model is trained on the full training set.")
        self.write_contents("The scores are computed on the full test set.\n\n")
        self.write_contents(report)

    def report_environtment(self):
        """
        Reports local environment info.
        """

        system = platform.system()

        self.write_header("Environment")

        self.write_contents("OS Version          : {}\n".format(platform.platform()))
        self.write_contents("System              : {}\n".format(system))
        self.write_contents("Machine             : {}\n".format(platform.machine()))

        if system == "Windows":  # pragma: no cover
            memory = subprocess.check_output(
                "wmic memorychip get capacity", shell=True, universal_newlines=True
            )
            total_mem = 0
            for m in memory.split("  \r\n")[1:-1]:
                total_mem += int(m)

            self.write_contents(
                "Processor           : {}\n".format(platform.processor())
            )
            self.write_contents(
                "Memory              : {:.2f} GB\n".format(
                    round(total_mem / (1024 ** 2))
                )
            )
        elif platform.system() == "Darwin":  # pragma: no cover
            processor = subprocess.check_output(
                "/usr/sbin/sysctl -n machdep.cpu.brand_string",
                shell=True,
                universal_newlines=True,
            )
            memory = subprocess.check_output(
                "sysctl -n hw.memsize", shell=True, universal_newlines=True
            )
            self.write_contents("Processor           : {}\n".format(processor.strip()))
            self.write_contents(
                "Memory              : {}\n".format(
                    round(int(memory.strip()) / (1024 ** 2))
                )
            )
        elif platform.system() == "Linux":
            processor = subprocess.check_output(
                "cat /proc/cpuinfo | grep 'model name'",
                shell=True,
                universal_newlines=True,
            )
            with open("/proc/meminfo", "r") as f:
                lines = f.readlines()

            memory = [int(s) for s in lines[0].split() if s.isdigit()][0]
            processor = processor.replace("\t", "").split("model name:")[1].strip()
            self.write_contents("Processor           : {}\n".format(processor))
            self.write_contents(
                "Memory              : {:.2f} GB\n".format(round(memory / (1024 ** 2)))
            )

        self.write_contents("CPU Count           : {}\n".format(os.cpu_count()))
        self.write_contents(
            "Python Version      : {}\n".format(platform.python_version())
        )
        self.write_contents(
            "Python Compiler     : {}\n".format(platform.python_compiler())
        )
        self.write_contents(
            "Python Build        : {}\n".format(platform.python_build())
        )

    def write_image(self, name):  # pragma: no cover

        if self.docx:
            self.doc.add_picture(
                os.path.join(self.image_dir, name), width=Inches(6), height=Inches(4)
            )

            self.doc.save(self.docx_filename)

    def write_metrics(self, df: pd.DataFrame):

        self.write_header("Metrics", level=2)
        self.write_contents(df.to_string())

        if self.docx:  # pragma: no cover
            t = self.doc.add_table(df.shape[0] + 1, df.shape[1])

            t.style = "Light List Accent 1"

            # add the header rows.
            for j in range(df.shape[-1]):
                t.cell(0, j).text = df.columns[j]

            # add the rest of the data frame
            for i in range(df.shape[0]):
                for j in range(df.shape[-1]):
                    t.cell(i + 1, j).text = str(df.values[i, j])

            self.doc.save(self.docx_filename)



"""
This file contains the following methods:

replace_missing_new_category
replace_missing_remove_row
"""

import numpy as np
import pandas as pd
from aethos.util import _get_columns


def replace_missing_new_category(
    x_train, x_test=None, col_to_category=None, constant=None
):
    """
    Replaces missing values in categorical column with its own category. The categories can be autochosen
    from the defaults set.
    
    Parameters
    ----------
    x_train : DataFrame
        Dataset
        
    x_test : DataFrame
        Testing Dataset, by default None
        
    col_to_category : list or dict, optional
        A dictionary mapping column name to the category name you want to replace , by default None

    constant : str, int or float, optional
        Category placeholder value for missing values, by default None
    
    Returns
    -------
    Dataframe, *Dataframe:
        Cleaned columns of the Dataframe(s) provides with the provided constant.
        
    Returns 2 Dataframes if x_test is provided.

    Examples
    --------
    >>> ReplaceMissingCategory({'a': "Green", 'b': "Canada", 'c': "December"})
    >>> ReplaceMissingCategory("Blue", ['a', 'b', 'c'])
    """

    if isinstance(col_to_category, list):
        col_to_category = _get_columns(col_to_category, x_train)

    str_missing_categories = ["Other", "Unknown", "Missingx_trainCategory"]
    num_missing_categories = [-1, -999, -9999]

    if isinstance(col_to_category, dict):

        for col in col_to_category.keys():
            x_train[col].fillna(col_to_category[col], inplace=True)

            if x_test is not None:
                x_test[col].fillna(col_to_category[col], inplace=True)

    elif isinstance(col_to_category, list) and constant is not None:

        for col in col_to_category:
            x_train[col].fillna(constant, inplace=True)

            if x_test is not None:
                x_test[col].fillna(constant, inplace=True)

    else:

        for col in col_to_category:
            # Check if column is a number
            if np.issubdtype(x_train[col].dtype, np.number):
                new_category_name = _determine_default_category(
                    x_train, col, num_missing_categories
                )
                x_train[col].fillna(new_category_name, inplace=True)

                # Convert numeric categorical column to integer
                x_train[col] = x_train[col].astype(int)

                if x_test is not None:
                    x_test[col].fillna(new_category_name, inplace=True)
                    # Convert numeric categorical column to integer
                    x_test[col] = x_test[col].astype(int)
            else:
                new_category_name = _determine_default_category(
                    x_train, col, str_missing_categories
                )
                x_train[col].fillna(new_category_name, inplace=True)

                if x_test is not None:
                    new_category_name = _determine_default_category(
                        x_train, col, str_missing_categories
                    )
                    x_test[col].fillna(new_category_name, inplace=True)

    return x_train, x_test


def replace_missing_remove_row(x_train, x_test=None, cols_to_remove=[]):
    """
    Remove rows where the value of a column for those rows is missing.
        
    Parameters
    ----------
    x_train : DataFrame
        Dataset
        
    x_test : DataFrame
        Testing Dataset, by default None

    cols_to_remove : list
        List of columns you want to check to see if they have missing values in a row

    Returns
    -------
    Dataframe, *Dataframe:
        Cleaned columns of the Dataframe(s) provides with the provided constant.
        
    Returns 2 Dataframes if x_test is provided.
    """

    x_train = x_train.dropna(axis=0, subset=cols_to_remove)

    if x_test is not None:
        x_test = x_test.dropna(axis=0, subset=cols_to_remove)

    return x_train, x_test


def _determine_default_category(x_train, col, replacement_categories):
    """
    A utility function to help determine the default category name for a column that has missing
    categorical values. 
    
    It takes in a list of possible values and if any the first value in the list
    that is not a value in the column is the category that will be used to replace missing values.
    """

    unique_vals_col = x_train[col].unique()
    for potential_category in replacement_categories:

        # If the potential category is not already a category, it becomes the default missing category
        if potential_category not in unique_vals_col:
            new_category_name = potential_category
            break

    return new_category_name



import os
import yaml
from IPython import get_ipython

pkg_directory = os.path.dirname(__file__)

with open(
    os.path.join(os.path.expanduser("~"), ".aethos", "config.yml"), "r"
) as ymlfile:
    cfg = yaml.safe_load(ymlfile)

with open(f"{pkg_directory}/technique_reasons.yml", "r") as stream:
    technique_reason_repo = yaml.safe_load(stream)

shell = get_ipython().__class__.__name__

DEFAULT_MODEL_DIR = os.path.join(os.path.expanduser("~"), ".aethos", "models")
DEFAULT_REPORT_DIR = os.path.join(os.path.expanduser("~"), ".aethos", "reports")
DEFAULT_IMAGE_DIR = os.path.join(os.path.expanduser("~"), ".aethos", "images")



import numpy as np

penalty_12 = ["l1", "l2"]
penalty_12none = ["l1", "l2", None]
penalty_12e = ["l1", "l2", "elasticnet"]
penalty_all = ["l1", "l2", None, "elasticnet"]
max_iter = [100, 300, 1000]
max_iter_inf = [100, 300, 500, 1000, np.inf]
max_iter_inf2 = [100, 300, 500, 1000, -1]
tol = [1e-4, 1e-3, 1e-2]
warm_start = [True, False]
alpha = [1e-5, 1e-4, 1e-3, 1e-2, 0.1, 1, 3, 10]
alpha_small = [1e-5, 1e-3, 0.1, 1]
n_iter = [5, 10, 20]
eta0 = [1e-4, 1e-3, 1e-2, 0.1]
C = [1e-2, 0.1, 1, 5, 10]
C_small = [0.1, 1, 5]
epsilon = [1e-3, 1e-2, 0.1, 0]
normalize = [True, False]
kernel = ["linear", "poly", "rbf", "sigmoid"]
degree = [1, 2, 3, 4, 5]
gamma = list(np.logspace(-9, 3, 6)) + ["auto"]
gamma_small = list(np.logspace(-6, 3, 3)) + ["auto"]
coef0 = [0, 0.1, 0.3, 0.5, 0.7, 1]
coef0_small = [0, 0.4, 0.7, 1]
shrinking = [True, False]
nu = [1e-4, 1e-2, 0.1, 0.3, 0.5, 0.75, 0.9]
nu_small = [1e-2, 0.1, 0.5, 0.9]
n_neighbors = [5, 7, 10, 15, 20]
neighbor_algo = ["ball_tree", "kd_tree", "brute"]
neighbor_leaf_size = [1, 2, 5, 10, 20, 30, 50, 100]
neighbor_metric = ["cityblock", "euclidean", "l1", "l2", "manhattan"]
neighbor_radius = [1e-2, 0.1, 1, 5, 10]
learning_rate = ["constant", "invscaling", "adaptive"]
learning_rate_small = ["invscaling", "adaptive"]
n_estimators = [2, 3, 5, 10, 25, 50, 100]
n_estimators_small = [2, 10, 25, 100]
max_features = [3, 5, 10, 25, 50, "auto", "log2", None]
max_features_small = [3, 5, 10, "auto", "log2", None]
max_depth = [None, 3, 5, 7, 10]
max_depth_small = [None, 5, 10]
min_samples_split = [2, 5, 10, 0.1]
min_impurity_split = [1e-7, 1e-6, 1e-5, 1e-4, 1e-3]
tree_learning_rate = [0.8, 1]
min_samples_leaf = [2]

logreg_gridsearch = {
    "penalty": penalty_12,
    "max_iter": max_iter,
    "tol": tol,
    "warm_start": warm_start,
    "C": C,
    "solver": ["liblinear"],
}



import os


def _create_project_dir(project_dir: str, name: str):
    """
    Creates the projects directory.
    
    Parameters
    ----------
    project_dir : str
        Full path of the project dir.

    name : str
        Name of the project
    """

    os.system(f"mkdir -p {project_dir}/{name}/app")



import os
from pathlib import Path

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import pandas_bokeh
import plotly.express as px
import ptitprince as pt
import seaborn as sns
from aethos.config import DEFAULT_IMAGE_DIR, cfg
from aethos.util import _make_dir
from aethos.visualizations.util import _make_image_dir
from bokeh.io import export_png
from scipy import stats


def raincloud(col: str, target_col: str, data: pd.DataFrame, output_file="", **params):
    """
    Visualizes 2 columns using raincloud.
    
    Parameters
    ----------
    col : str
        Column name of general data

    target_col : str
        Column name of measurable data, numerical

    data : Dataframe
        Dataframe of the data

    params: dict
        Parameters for the RainCloud visualization

    ouput_file : str
        Output file name for the image including extension (.jpg, .png, etc.)
    """

    fig, ax = plt.subplots(figsize=(12, 8))

    if not params:
        params = {
            "pointplot": True,
            "width_viol": 0.8,
            "width_box": 0.4,
            "orient": "h",
            "move": 0.0,
            "ax": ax,
        }

    ax = pt.RainCloud(x=col, y=target_col, data=data.infer_objects(), **params)

    if output_file:  # pragma: no cover
        img_dir = _make_image_dir()
        fig.savefig(os.path.join(img_dir, output_file))


def barplot(
    x,
    y,
    data,
    method=None,
    orient="v",
    stacked=False,
    output_file="",
    **barplot_kwargs,
):
    """
    Visualizes a bar plot.

    Kwargs are bokeh plot, vbar, and hbar bokeh plots.
    
    Parameters
    ----------
    x : str
        Column name for the x axis.

    y : list
        Columns for the y axis

    data : Dataframe
        Dataset

    method : str
        Method to aggregate groupy data
        Examples: min, max, mean, etc., optional
        by default None

    orient : str, optional
        Orientation of graph, 'h' for horizontal
        'v' for vertical, by default 'v'

    stacked : bool
        Whether to stack the different columns resulting in a stacked bar chart,
        by default False
    """

    alpha = barplot_kwargs.pop("alpha", 0.6)
    y.append(x)

    data_copy = data[y].copy()
    data_copy = data_copy.groupby(x, as_index=False)
    data_copy = getattr(data_copy, method)()
    data_copy = data_copy.set_index(x)

    if orient == "v":

        p_bar = data_copy.plot_bokeh.bar(
            stacked=stacked, alpha=alpha, **barplot_kwargs,
        )

    else:
        p_bar = data_copy.plot_bokeh.barh(
            stacked=stacked, alpha=alpha, **barplot_kwargs
        )

    if output_file:  # pragma: no cover
        img_dir = _make_image_dir()

        if Path(output_file).suffix == ".html":
            pandas_bokeh.output_file(os.path.join(img_dir, output_file))
        else:
            export_png(p_bar, os.path.join(img_dir, output_file))


def scatterplot(
    x: str,
    y: str,
    z=None,
    data=None,
    category=None,
    title="Scatter Plot",
    size=8,
    output_file="",
    **scatterplot_kwargs,
):
    """
    Plots a scatter plot.
    
    Parameters
    ----------
    x : str
        X axis column

    y : str
        Y axis column

    z: str
        Z axis column

    data : Dataframe
        Dataframe

    category : str, optional
        Category to group your data, by default None

    title : str, optional
        Title of the plot, by default 'Scatterplot'

    size : int or str, optional
        Size of the circle, can either be a number
        or a column name to scale the size, by default 8

    output_file : str, optional
        If a name is provided save the plot to an html file, by default ''
    """

    if z is None:
        fill_alpha = scatterplot_kwargs.pop("fill_alpha", 0.6)
        data[[x, y]] = data[[x, y]].apply(pd.to_numeric)

        p_scatter = data.plot_bokeh.scatter(
            x=x,
            y=y,
            category=category,
            title=title,
            size=size,
            fill_alpha=fill_alpha,
            **scatterplot_kwargs,
        )

        if output_file:  # pragma: no cover
            img_dir = _make_image_dir()

            if Path(output_file).suffix == ".html":
                pandas_bokeh.output_file(os.path.join(img_dir, output_file))
            else:
                export_png(p_scatter, os.path.join(img_dir, output_file))

    else:
        fig = px.scatter_3d(data, x=x, y=y, z=z, **scatterplot_kwargs)

        if output_file:  # pragma: no cover
            img_dir = _make_image_dir()
            fig.write_image(os.path.join(img_dir, output_file))

        fig.show()


def lineplot(
    x: str, y: list, data, title="Line Plot", output_file="", **lineplot_kwargs
):
    """
    Plots a line plot.
    
    Parameters
    ----------
    x : str
        X axis column

    y : list
        Y axis column

    data : Dataframe
        Dataframe

    title : str, optional
        Title of the plot, by default 'Line Plot'

    output_file : str, optional
        If a name is provided save the plot to an html file, by default ''
    """

    y.append(x)
    data_copy = data[y].copy()
    data_copy = data_copy.set_index(x)
    xlabel = lineplot_kwargs.pop("xlabel", x)

    p_line = data_copy.plot_bokeh.line(title=title, xlabel=xlabel, **lineplot_kwargs)

    if output_file:  # pragma: no cover
        img_dir = _make_image_dir()

        if Path(output_file).suffix == ".html":
            pandas_bokeh.output_file(os.path.join(img_dir, output_file))
        else:
            export_png(p_line, os.path.join(img_dir, output_file))


def correlation_matrix(
    df, data_labels=False, hide_mirror=False, output_file="", **kwargs
):
    """
    Plots a correlation matrix.
    
    Parameters
    ----------
    df : DataFrame
        Data

    data_labels : bool, optional
        Whether to display the correlation values, by default False

    hide_mirror : bool, optional
        Whether to display the mirroring half of the correlation plot, by default False

    ouput_file : str
        Output file name for the image including extension (.jpg, .png, etc.)
    """

    corr = df.corr()

    fig, ax = plt.subplots(figsize=(11, 9))

    if hide_mirror:
        mask = np.zeros_like(corr, dtype=np.bool)
        mask[np.triu_indices_from(mask)] = True
    else:
        mask = None

    cmap = sns.diverging_palette(220, 10, as_cmap=True)

    sns.heatmap(
        corr,
        cmap=cmap,
        vmax=0.3,
        center=0,
        square=True,
        mask=mask,
        annot=data_labels,
        fmt=".2f",
        linewidths=0.5,
        cbar_kws={"shrink": 0.5},
        **kwargs,
    )

    if output_file:  # pragma: no cover
        img_dir = _make_image_dir()
        fig.savefig(os.path.join(img_dir, output_file))


# TODO: Make pair plots customizable using PairGrid
def pairplot(
    df, kind="scatter", diag_kind="auto", hue=None, output_file=None, **kwargs
):
    """
    Plots pairplots of the variables in the DataFrame
    
    Parameters
    ----------
    df : DataFrame
        Data

    kind : {'scatter', 'reg'}, optional
        Type of plot for off-diag plots, by default 'scatter'

    diag_kind : {'auto', 'hist', 'kde'}, optional
        Type of plot for diagonal, by default 'auto'

    hue : str, optional
        Column to colour points by, by default None

    ouput_file : str
        Output file name for the image including extension (.jpg, .png, etc.)
    """

    palette = kwargs.pop("color", sns.color_palette("pastel"))

    g = sns.pairplot(
        df, kind=kind, diag_kind=diag_kind, hue=hue, palette=palette, **kwargs
    )

    if output_file:  # pragma: no cover
        img_dir = _make_image_dir()
        g.savefig(os.path.join(img_dir, output_file))


def jointplot(x, y, df, kind="scatter", output_file="", **kwargs):
    """
    Plots a joint plot of 2 variables.
    
    Parameters
    ----------
    x : str
        X axis column

    y : str
        y axis column

    df : DataFrame
        Data

    kind : { “scatter” | “reg” | “resid” | “kde” | “hex” }, optional
        Kind of plot to draw, by default 'scatter'

    ouput_file : str
        Output file name for the image including extension (.jpg, .png, etc.)
    """

    # NOTE: Ignore the deprecation warning for showing the R^2 statistic until Seaborn reimplements it
    import warnings

    warnings.simplefilter("ignore", UserWarning)

    sns.set(style="ticks", color_codes=True)
    color = kwargs.pop("color", "crimson")

    g = sns.jointplot(x=x, y=y, data=df, kind=kind, color=color, **kwargs).annotate(
        stats.pearsonr
    )

    if output_file:  # pragma: no cover
        img_dir = _make_image_dir()
        g.savefig(os.path.join(img_dir, output_file))


def histogram(x: list, data: pd.DataFrame, output_file="", **kwargs):
    """
    Plots a histogram.
    
    Parameters
    ----------
    x : list
        Columns to plot histogram for.

    data : pd.DataFrame
        Dataframe of the data.

    ouput_file : str
        Output file name for the image including extension (.jpg, .png, etc.)
    """

    sns.set(style="ticks", color_codes=True)
    sns.set_palette(sns.color_palette("pastel"))

    if len(x) == 1:
        g = sns.distplot(data[x], rug=True, **kwargs)

    else:
        for col in x:
            g = sns.distplot(data[col], label=col, **kwargs)

        plt.legend()

    if output_file:  # pragma: no cover
        img_dir = _make_image_dir()
        g.figure.savefig(os.path.join(img_dir, output_file))



import pandas as pd
from aethos.util import _numeric_input_conditions, drop_replace_columns
from sklearn.preprocessing import PolynomialFeatures
import numpy as np


def polynomial_features(x_train, x_test=None, list_of_cols=[], **poly_kwargs):
    """
    Computes polynomial features from your existing features.
    
    Parameters
    ----------
    x_train : DataFrame
        Dataset
        
    x_test : DataFrame
        Testing dataset, by default None

    list_of_cols : list, optional
        A list of specific columns to apply this technique to
        If `list_of_cols` is not provided, the strategy will be
        applied to all numeric columns, by default []

    keep_col : bool, optional
        True to not remove the columns, by default False

    poly_kwargs : dict or kwargs
        Polynomial Features constructor key word arguments
    
    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows normalized.

    Returns 2 Dataframes if x_test is provided.
    """

    poly = PolynomialFeatures(**poly_kwargs)
    list_of_cols = _numeric_input_conditions(list_of_cols, x_train)

    scaled_data = poly.fit_transform(x_train[list_of_cols])
    scaled_df = pd.DataFrame(scaled_data, columns=poly.get_feature_names())
    x_train = drop_replace_columns(x_train, list_of_cols, scaled_df)

    if x_test is not None:
        scaled_x_test = poly.transform(x_test)
        scaled_test_df = pd.DataFrame(scaled_x_test, columns=poly.get_feature_names())
        x_test = drop_replace_columns(x_test, list_of_cols, scaled_test_df)

    return x_train, x_test


def drop_correlated_features(x_train, x_test=None, threshold=0.95):
    """
    Drops highly correlated features.

    Derived from: https://chrisalbon.com/machine_learning/feature_selection/drop_highly_correlated_features/
    
    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Training dataset, by default None.
        
    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.

    threshold : float, optional
        Correlation coefficient value threshold to drop the feature

    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific column are missing

    Returns 2 Dataframes if x_test is provided.  
    """

    corr = x_train.corr().abs()
    upper = corr.where(np.triu(np.ones(corr.shape), k=1).astype(np.bool))
    drop_cols = [col for col in upper.columns if any(upper[col] > 0.95)]

    x_train.drop(drop_cols, axis=1, inplace=True)

    if x_test is not None:
        x_test.drop(drop_cols, axis=1, inplace=True)

    return x_train, x_test



xgb_reg_doc = """
Trains an XGBoost Regression Model.

XGBoost is an optimized distributed gradient boosting library designed to be highly efficient, flexible and portable.
It implements machine learning algorithms under the Gradient Boosting framework.
XGBoost provides a parallel tree boosting (also known as GBDT, GBM) that solve many data science problems in a fast and accurate way.
The same code runs on major distributed environment (Hadoop, SGE, MPI) and can solve problems beyond billions of examples.

For more XGBoost info, you can view it here: https://xgboost.readthedocs.io/en/latest/ and
https://github.com/dmlc/xgboost/blob/master/doc/parameter.rst. 

If running cross-validation, the implemented cross validators are:
    - 'kfold' for KFold
    - 'strat-kfold' for StratifiedKfold

For more information regarding the cross validation methods, you can view them here: https://scikit-learn.org/stable/modules/classes.html#module-sklearn.model_selection 

Possible scoring metrics: 
    - ‘explained_variance’ 	 
    - ‘max_error’ 	 
    - ‘neg_mean_absolute_error’ --> MAE 
    - ‘neg_mean_squared_error’ --> MSE
    - ‘neg_mean_squared_log_error’ --> MSLE 
    - ‘neg_median_absolute_error’ --> MeAE 
    - ‘r2’

Parameters
----------
cv : bool, optional
    If True run crossvalidation on the model, by default None.

gridsearch : int, Crossvalidation Generator, optional
    Cross validation method, by default None

score : str, optional
    Scoring metric to evaluate models, by default 'neg_mean_squared_error'

model_name : str, optional
    Name for this model, by default "xgb_reg"

new_col_name : str, optional
    Name of column for labels that are generated, by default "xgb_reg_predictions"

run : bool, optional
    Whether to train the model or just initialize it with parameters (useful when wanting to test multiple models at once) , by default False

verbose : bool, optional
    True if you want to print out detailed info about the model training, by default False    	

max_depth : int
    Maximum tree depth for base learners. By default 3

learning_rate : float
    Boosting learning rate (xgb's "eta"). By default 0.1

n_estimators : int
    Number of trees to fit. By default 100.

objective : string or callable
    Specify the learning task and the corresponding learning objective or
    a custom objective function to be used (see note below).
    By default, reg:linear

booster: string
    Specify which booster to use: gbtree, gblinear or dart. By default 'gbtree'

tree_method: string
    Specify which tree method to use
    If this parameter is set to default, XGBoost will choose the most conservative option
    available.  It's recommended to study this option from parameters
    document. By default 'auto'

gamma : float
    Minimum loss reduction required to make a further partition on a leaf node of the tree.
    By default 0

subsample : float
    Subsample ratio of the training instance.
    By default 1

reg_alpha : float (xgb's alpha)
    L1 regularization term on weights. By default 0

reg_lambda : float (xgb's lambda)
    L2 regularization term on weights. By default 1

scale_pos_weight : float
    Balancing of positive and negative weights. By default 1

base_score:
    The initial prediction score of all instances, global bias. By default 0

missing : float, optional
    Value in the data which needs to be present as a missing value. If
    None, defaults to np.nan.
    By default, None

num_parallel_tree: int
    Used for boosting random forest.
    By default 1

importance_type: string, default "gain"
    The feature importance type for the feature_importances\\_ property:
    either "gain", "weight", "cover", "total_gain" or "total_cover".
    By default 'gain'.

Note
----
A custom objective function can be provided for the ``objective``
parameter. In this case, it should have the signature
``objective(y_true, y_pred) -> grad, hess``:

y_true: array_like of shape [n_samples]
    The target values

y_pred: array_like of shape [n_samples]
    The predicted values

grad: array_like of shape [n_samples]
    The value of the gradient for each sample point.

hess: array_like of shape [n_samples]
    The value of the second derivative for each sample point

Returns
-------
RegressionModel
    RegressionModel object to view results and analyze results
"""


lgbm_reg_doc = """
Trains an LightGBM Regression Model.

LightGBM is a gradient boosting framework that uses a tree based learning algorithm.

Light GBM grows tree vertically while other algorithm grows trees horizontally meaning that Light GBM grows tree leaf-wise while other algorithm grows level-wise.
It will choose the leaf with max delta loss to grow.
When growing the same leaf, Leaf-wise algorithm can reduce more loss than a level-wise algorithm.

For more LightGBM info, you can view it here: https://github.com/microsoft/LightGBM and
https://lightgbm.readthedocs.io/en/latest/pythonapi/lightgbm.LGBMRegressor.html#lightgbm.LGBMRegressor

If running cross-validation, the implemented cross validators are:
    - 'kfold' for KFold
    - 'strat-kfold' for StratifiedKfold

For more information regarding the cross validation methods, you can view them here: https://scikit-learn.org/stable/modules/classes.html#module-sklearn.model_selection 

Possible scoring metrics: 
    - ‘explained_variance’ 	 
    - ‘max_error’ 	 
    - ‘neg_mean_absolute_error’ --> MAE 
    - ‘neg_mean_squared_error’ --> MSE
    - ‘neg_mean_squared_log_error’ --> MSLE 
    - ‘neg_median_absolute_error’ --> MeAE 
    - ‘r2’

Parameters
----------
cv : bool, optional
    If True run crossvalidation on the model, by default None.

gridsearch : int, Crossvalidation Generator, optional
    Cross validation method, by default None

score : str, optional
    Scoring metric to evaluate models, by default 'neg_mean_squared_error'

model_name : str, optional
    Name for this model, by default "lgbm_reg"

new_col_name : str, optional
    Name of column for labels that are generated, by default "lgbm_reg_predictions"

run : bool, optional
    Whether to train the model or just initialize it with parameters (useful when wanting to test multiple models at once) , by default False

verbose : bool, optional
    True if you want to print out detailed info about the model training, by default False    	

boosting_type (string, optional (default='gbdt'))
    ‘gbdt’, traditional Gradient Boosting Decision Tree. ‘dart’, Dropouts meet Multiple Additive Regression Trees. ‘goss’, Gradient-based One-Side Sampling. ‘rf’, Random Forest.

num_leaves (int, optional (default=31))
    Maximum tree leaves for base learners.

max_depth (int, optional (default=-1))
    Maximum tree depth for base learners, <=0 means no limit.

learning_rate (float, optional (default=0.1))
    Boosting learning rate. You can use callbacks parameter of fit method to shrink/adapt learning rate in training using reset_parameter callback. Note, that this will ignore the learning_rate argument in training.

n_estimators (int, optional (default=100))
    Number of boosted trees to fit.

subsample_for_bin (int, optional (default=200000))
    Number of samples for constructing bins.

objective (string, callable or None, optional (default=None))
    Specify the learning task and the corresponding learning objective or a custom objective function to be used (see note below). Default: ‘regression’ for LGBMRegressor, ‘binary’ or ‘multiclass’ for LGBMClassifier, ‘lambdarank’ for LGBMRanker.

class_weight (dict, 'balanced' or None, optional (default=None))
    Weights associated with classes in the form {class_label: weight}. Use this parameter only for multi-class classification task; for binary classification task you may use is_unbalance or scale_pos_weight parameters. Note, that the usage of all these parameters will result in poor estimates of the individual class probabilities. You may want to consider performing probability calibration (https://scikit-learn.org/stable/modules/calibration.html) of your model. The ‘balanced’ mode uses the values of y to automatically adjust weights inversely proportional to class frequencies in the input data as n_samples / (n_classes * np.bincount(y)). If None, all classes are supposed to have weight one. Note, that these weights will be multiplied with sample_weight (passed through the fit method) if sample_weight is specified.

min_split_gain (float, optional (default=0.))
    Minimum loss reduction required to make a further partition on a leaf node of the tree.

min_child_weight (float, optional (default=1e-3))
    Minimum sum of instance weight (hessian) needed in a child (leaf).

min_child_samples (int, optional (default=20))
    Minimum number of data needed in a child (leaf).

subsample (float, optional (default=1.))
    Subsample ratio of the training instance.

subsample_freq (int, optional (default=0))
    Frequence of subsample, <=0 means no enable.

colsample_bytree (float, optional (default=1.))
    Subsample ratio of columns when constructing each tree.

reg_alpha (float, optional (default=0.))
    L1 regularization term on weights.

reg_lambda (float, optional (default=0.))
    L2 regularization term on weights.

random_state (int or None, optional (default=None))
    Random number seed. If None, default seeds in C++ code will be used.

n_jobs (int, optional (default=-1))
    Number of parallel threads.

silent (bool, optional (default=True))
    Whether to print messages while running boosting.

importance_type (string, optional (default='split'))
    The type of feature importance to be filled into feature_importances_. If ‘split’, result contains numbers of times the feature is used in a model. If ‘gain’, result contains total gains of splits which use the feature.
    
Returns
-------
RegressionModel
    RegressionModel object to view results and analyze results
"""


catboost_reg_doc = """
Trains an CatBoost Regression Model.

CatBoost is an algorithm for gradient boosting on decision trees. 

For more CatBoost info, you can view it here: https://catboost.ai/docs/concepts/about.html and
https://catboost.ai/docs/concepts/python-reference_parameters-list.html#python-reference_parameters-list

If running cross-validation, the implemented cross validators are:
- 'kfold' for KFold
- 'strat-kfold' for StratifiedKfold

For more information regarding the cross validation methods, you can view them here: https://scikit-learn.org/stable/modules/classes.html#module-sklearn.model_selection 

Possible scoring metrics: 
- ‘explained_variance’ 	 
- ‘max_error’ 	 
- ‘neg_mean_absolute_error’ --> MAE 
- ‘neg_mean_squared_error’ --> MSE
- ‘neg_mean_squared_log_error’ --> MSLE 
- ‘neg_median_absolute_error’ --> MeAE 
- ‘r2’

Parameters
----------
cv : bool, optional
If True run crossvalidation on the model, by default None.

gridsearch : int, Crossvalidation Generator, optional
Cross validation method, by default None

score : str, optional
Scoring metric to evaluate models, by default 'neg_mean_squared_error'

model_name : str, optional
Name for this model, by default "cb_reg"

new_col_name : str, optional
Name of column for labels that are generated, by default "cb_reg_predictions"

run : bool, optional
Whether to train the model or just initialize it with parameters (useful when wanting to test multiple models at once) , by default False

verbose : bool, optional
True if you want to print out detailed info about the model training, by default False    	


Important Params
----------------

- cat_features
- one_hot_max_size
- learning_rate        
- n_estimators
- max_depth
- subsample
- colsample_bylevel
- colsample_bytree
- colsample_bynode
- l2_leaf_reg
- random_strength

For more parameter information (as there is a lot) please view https://catboost.ai/docs/concepts/python-reference_parameters-list.html#python-reference_parameters-list

Returns
-------
RegressionModel
RegressionModel object to view results and analyze results
"""



"""
This file contains the following functions:

pca
apply
"""

import pandas as pd
import swifter
from sklearn.decomposition import PCA


def pca(x_train, x_test=None, **pca_kwargs):
    """
    Performs Principal Component Analysis on a dataset.
    
    Parameters
    ----------
    x_train : DataFrame
        Dataset

    x_test : DataFrame
        Testing dataset, by default None

    pca_kwargs : dict or key word args
        PCA properties
    
    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with the new column

    Returns 2 Dataframes if x_test is provided. 
    """

    pca = PCA(**pca_kwargs)

    x_train = pd.DataFrame(pca.fit_transform(x_train))

    if x_test is not None:
        x_test = pd.DataFrame(pca.transform(x_test))

    return x_train, x_test


def apply(x_train, func, output_col: str, x_test=None):
    """
    Wrapper for pandas apply function to be used in this library. Applies `func` to the entire data
    or just the trianing and testing data

    Parameters
    ----------
    x_train : DataFrame
        Dataset

    func : Function pointer
        Function describing the transformation for the new column

    output_col : str
        New column name
        
    x_test : DataFrame
        Testing dataset, by default None
    
    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with the new column

    Returns 2 Dataframes if x_test is provided. 
    """

    x_train.loc[:, output_col] = x_train.swifter.progress_bar().apply(func, axis=1)

    if x_test is not None:
        x_test.loc[:, output_col] = x_test.swifter.progress_bar().apply(func, axis=1)

    return x_train, x_test



__all__ = [
    "config",
    "get_option",
    "set_option",
    "reset_option",
    "describe_option",
    "options",
    "cfg",
    "shell",
    "technique_reason_repo",
]

from . import config

from .config import (
    describe_option,
    get_option,
    options,
    reset_option,
    set_option,
)

from .user_config import *



from aethos.visualizations.visualize import *


class Visualizations(object):
    def __init__(self, data):
        self.data = data

    @property
    def plot_colors(self):  # pragma: no cover
        """
        Displays all plot colour names
        """

        from IPython.display import IFrame

        IFrame(
            "https://python-graph-gallery.com/wp-content/uploads/100_Color_names_python.png"
        )

    @property
    def plot_colorpalettes(self):  # pragma: no cover
        """
        Displays color palette configuration guide.
        """

        from IPython.display import IFrame

        IFrame("https://seaborn.pydata.org/tutorial/color_palettes.html")

    def raincloud(self, x=None, y=None, output_file="", **params):
        """
        Combines the box plot, scatter plot and split violin plot into one data visualization.
        This is used to offer eyeballed statistical inference, assessment of data distributions (useful to check assumptions),
        and the raw data itself showing outliers and underlying patterns.

        A raincloud is made of:
        1) "Cloud", kernel desity estimate, the half of a violinplot.
        2) "Rain", a stripplot below the cloud
        3) "Umberella", a boxplot
        4) "Thunder", a pointplot connecting the mean of the different categories (if `pointplot` is `True`)

        Useful parameter documentation
        ------------------------------
        https://seaborn.pydata.org/generated/seaborn.boxplot.html

        https://seaborn.pydata.org/generated/seaborn.violinplot.html

        https://seaborn.pydata.org/generated/seaborn.stripplot.html

        Parameters
        ----------
        x : str
            X axis data, reference by column name, any data

        y : str
            Y axis data, reference by column name, measurable data (numeric)
            by default target_field

        hue : Iterable, np.array, or dataframe column name if 'data' is specified
            Second categorical data. Use it to obtain different clouds and rainpoints

        output_file : str, optional
            Output file name for image with extension (i.e. jpeg, png, etc.)

        orient : str                  
            vertical if "v" (default), horizontal if "h"

        width_viol : float            
            width of the cloud

        width_box : float             
            width of the boxplot

        palette : list or dict        
            Colours to use for the different levels of categorical variables

        bw : str or float
            Either the name of a reference rule or the scale factor to use when computing the kernel bandwidth,
            by default "scott"

        linewidth : float             
            width of the lines

        cut : float
            Distance, in units of bandwidth size, to extend the density past the extreme datapoints.
            Set to 0 to limit the violin range within the range of the observed data,
            by default 2

        scale : str
            The method used to scale the width of each violin.
            If area, each violin will have the same area.
            If count, the width of the violins will be scaled by the number of observations in that bin.
            If width, each violin will have the same width.
            By default "area"

        jitter : float, True/1
            Amount of jitter (only along the categorical axis) to apply.
            This can be useful when you have many points and they overlap,
            so that it is easier to see the distribution. You can specify the amount of jitter (half the width of the uniform random variable support),
            or just use True for a good default.

        move : float                  
            adjust rain position to the x-axis (default value 0.)

        offset : float                
            adjust cloud position to the x-axis

        color : matplotlib color
            Color for all of the elements, or seed for a gradient palette.

        ax : matplotlib axes
            Axes object to draw the plot onto, otherwise uses the current Axes.

        figsize : (int, int)    
            size of the visualization, ex (12, 5)

        pointplot : bool   
            line that connects the means of all categories, by default False

        dodge : bool 
            When hue nesting is used, whether elements should be shifted along the categorical axis.

        Source: https://micahallen.org/2018/03/15/introducing-raincloud-plots/
        
        Examples
        --------
        >>> data.raincloud('col1') # Will plot col1 values on the x axis and your target variable values on the y axis
        >>> data.raincloud('col1', 'col2') # Will plot col1 on the x and col2 on the y axis
        >>> data.raincloud('col1', 'col2', output_file='raincloud.png')
        """

        if y is None:
            y = self.target_field

        raincloud(y, x, self.data, output_file=output_file, **params)

        if output_file and self.report:  # pragma: no cover
            self.report.write_image(output_file)

    def barplot(
        self,
        x: str,
        y=None,
        method=None,
        orient="v",
        stacked=False,
        output_file="",
        **barplot_kwargs,
    ):
        """
        Plots a bar plot for the given columns provided using Bokeh.

        If `groupby` is provided, method must be provided for example you may want to plot Age against survival rate,
        so you would want to `groupby` Age and then find the `mean` as the method.

        For a list of group by methods please checkout the following pandas link:
        https://pandas.pydata.org/pandas-docs/stable/reference/groupby.html#computations-descriptive-stats

        For a list of possible arguments for the bar plot please checkout the following links:
        https://github.com/PatrikHlobil/Pandas-Bokeh#barplot and

        https://bokeh.pydata.org/en/latest/docs/reference/plotting.html#bokeh.plotting.figure.Figure.vbar or

        https://bokeh.pydata.org/en/latest/docs/reference/plotting.html#bokeh.plotting.figure.Figure.hbar for horizontal
        
        Parameters
        ----------
        x : str
            Column name for the x axis.

        y : list
            Columns you would like to see plotted against the x_col

        method : str
            Method to aggregate groupy data
            Examples: min, max, mean, etc., optional
            by default None

        orient : str, optional
            Orientation of graph, 'h' for horizontal
            'v' for vertical, by default 'v',

        stacked : bool
            Whether to stack the different columns resulting in a stacked bar chart,
            by default False

        output_file : str, optional
            Output html file name for image

        Examples
        --------
        >>> data.barplot(x='x', y=['y', 'z'], method='mean')
        >>> data.barplot(x='x', y=['y', 'z'], method='max', orient='h')
        """

        barplot(
            x,
            y,
            self.data,
            method=method,
            orient=orient,
            stacked=stacked,
            output_file=output_file,
            **barplot_kwargs,
        )

        if output_file and self.report:  # pragma: no cover
            self.report.write_image(output_file)

    def scatterplot(
        self,
        x=None,
        y=None,
        z=None,
        category=None,
        title="Scatter Plot",
        size=8,
        output_file="",
        **scatterplot_kwargs,
    ):
        """
        Plots a scatterplot for the given x and y columns provided using Bokeh.

        For a list of possible scatterplot_kwargs for 2 dimensional data please check out the following links:

        https://bokeh.pydata.org/en/latest/docs/reference/plotting.html#bokeh.plotting.figure.Figure.scatter

        https://bokeh.pydata.org/en/latest/docs/user_guide/styling.html#userguide-styling-line-properties 

        For more information on key word arguments for 3d data, please check them out here:

        https://www.plotly.express/plotly_express/#plotly_express.scatter_3d
        
        Parameters
        ----------
        x : str
            X column name

        y : str
            Y column name

        z : str
            Z column name, 

        category : str, optional
            Category to group your data, by default None

        title : str, optional
            Title of the plot, by default 'Scatter Plot'

        size : int or str, optional
            Size of the circle, can either be a number
            or a column name to scale the size, by default 8

        output_file : str, optional
            Output html file name for image

        **scatterplot_kwargs : optional
            See above links for list of possible scatterplot options.

        Examples
        --------
        >>> data.scatterplot(x='x', y='y') #2d
        >>> data.scatterplot(x='x', y='y', z='z') #3d
        >>> data.scatterplot(x='x', y='y', z='z', output_file='scatt')
        """

        scatterplot(
            x,
            y,
            z=z,
            data=self.data,
            title=title,
            category=category,
            size=size,
            output_file=output_file,
            **scatterplot_kwargs,
        )

        if output_file and self.report:  # pragma: no cover
            self.report.write_image(output_file)

    def lineplot(
        self, x: str, y: list, title="Line Plot", output_file="", **lineplot_kwargs
    ):
        """
        Plots a lineplot for the given x and y columns provided using Bokeh.

        For a list of possible lineplot_kwargs please check out the following links:

        https://github.com/PatrikHlobil/Pandas-Bokeh#lineplot

        https://bokeh.pydata.org/en/latest/docs/reference/plotting.html#bokeh.plotting.figure.Figure.line 
        
        Parameters
        ----------
        x : str
            X column name

        y : list
            Column names to plot on the y axis.

        title : str, optional
            Title of the plot, by default 'Line Plot'

        output_file : str, optional
            Output html file name for image

        color : str, optional
            Define a single color for the plot

        colormap : list or Bokeh color palette, optional
            Can be used to specify multiple colors to plot.
            Can be either a list of colors or the name of a Bokeh color palette : https://bokeh.pydata.org/en/latest/docs/reference/palettes.html

        rangetool : bool, optional
            If true, will enable a scrolling range tool.

        xlabel : str, optional
            Name of the x axis

        ylabel : str, optional
            Name of the y axis

        xticks : list, optional
            Explicitly set ticks on x-axis

        yticks : list, optional
            Explicitly set ticks on y-axis

        xlim : tuple (int or float), optional
            Set visible range on x axis

        ylim : tuple (int or float), optional
            Set visible range on y axis.

        **lineplot_kwargs : optional
            For a list of possible keyword arguments for line plot please see https://github.com/PatrikHlobil/Pandas-Bokeh#lineplot
            and https://bokeh.pydata.org/en/latest/docs/reference/plotting.html#bokeh.plotting.figure.Figure.line

        Examples
        --------
        >>> data.line_plot(x='x', y='y')
        >>> data.line_plot(x='x', y='y', output_file='line')
        """

        lineplot(
            x, y, self.data, title=title, output_file=output_file, **lineplot_kwargs,
        )

        if output_file and self.report:  # pragma: no cover
            self.report.write_image(output_file)

    def correlation_matrix(
        self, data_labels=False, hide_mirror=False, output_file="", **kwargs
    ):
        """
        Plots a correlation matrix of all the numerical variables.

        For more information on possible kwargs please see: https://seaborn.pydata.org/generated/seaborn.heatmap.html
        
        Parameters
        ----------
        data_labels : bool, optional
            True to display the correlation values in the plot, by default False

        hide_mirror : bool, optional
            Whether to display the mirroring half of the correlation plot, by default False

        output_file : str, optional
            Output file name for image with extension (i.e. jpeg, png, etc.)

        Examples
        --------
        >>> data.correlation_matrix(data_labels=True)
        >>> data.correlation_matrix(data_labels=True, output_file='corr.png')
        """

        correlation_matrix(
            self.data,
            data_labels=data_labels,
            hide_mirror=hide_mirror,
            output_file=output_file,
            **kwargs,
        )

        if output_file and self.report:  # pragma: no cover
            self.report.write_image(output_file)

    def pairplot(
        self, kind="scatter", diag_kind="auto", hue=None, output_file="", **kwargs
    ):
        """
        Plots pairplots of the variables from the training data.

        If hue is not provided and a target variable is set, the data will separated and highlighted by the classes in that column.

        For more info and kwargs on pair plots, please see: https://seaborn.pydata.org/generated/seaborn.pairplot.html
        
        Parameters
        ----------
        df : DataFrame
                Data

        kind : {'scatter', 'reg'}, optional
            Type of plot for off-diag plots, by default 'scatter'

        diag_kind : {'auto', 'hist', 'kde'}, optional
            Type of plot for diagonal, by default 'auto'

        hue : str, optional
            Column to colour points by, by default None

        {x, y}_vars : lists of variable names, optional
            Variables within data to use separately for the rows and columns of the figure; i.e. to make a non-square plot.

        palette : dict or seaborn color palette
            Set of colors for mapping the hue variable. If a dict, keys should be values in the hue variable.

        output_file : str, optional
            Output file name for image with extension (i.e. jpeg, png, etc.)

        Examples
        --------
        >>> data.pairplot(kind='kde')
        >>> data.pairplot(kind='kde', output_file='pair.png')
        """

        if self.target_field and not hue:
            hue = self.target_field
        elif not self.target_field and hue:
            hue = hue

        pairplot(
            self.data,
            kind=kind,
            diag_kind=diag_kind,
            hue=hue,
            output_file=output_file,
            **kwargs,
        )

        if output_file and self.report:  # pragma: no cover
            self.report.write_image(output_file)

    def jointplot(self, x: str, y: str, kind="scatter", output_file="", **kwargs):
        """
        Plots joint plots of 2 different variables.

        Scatter ('scatter'): Scatter plot and histograms of x and y.

        Regression ('reg'): Scatter plot, with regression line and histograms with kernel density fits.

        Residuals ('resid'): Scatter plot of residuals and histograms of residuals.

        Kernel Density Estimates ('kde'): Density estimate plot and histograms.

        Hex ('hex'): Replaces scatterplot with joint histogram using hexagonal bins and histograms on the axes.

        For more info and kwargs for joint plots, see https://seaborn.pydata.org/generated/seaborn.jointplot.html
        
        Parameters
        ----------
        x : str
            X axis column

        y : str
            y axis column

        kind : { “scatter” | “reg” | “resid” | “kde” | “hex” }, optional
            Kind of plot to draw, by default 'scatter'

        color : matplotlib color, optional
            Color used for the plot elements.

        dropna : bool, optional
            If True, remove observations that are missing from x and y.

        {x, y}lim : two-tuples, optional
            Axis limits to set before plotting.

        {joint, marginal, annot}_kws : dicts, optional
            Additional keyword arguments for the plot components.

        output_file : str, optional
            Output file name for image with extension (i.e. jpeg, png, etc.)

        Examples
        --------
        >>> data.jointplot(x='x', y='y', kind='kde', color='crimson')
        >>> data.jointplot(x='x', y='y', kind='kde', color='crimson', output_file='pair.png')
        """

        jointplot(x=x, y=y, df=self.data, kind=kind, output_file=output_file, **kwargs)

        if output_file and self.report:  # pragma: no cover
            self.report.write_image(output_file)

    def histogram(self, *x, output_file="", **kwargs):
        """
        Plots a histogram of the given column(s).

        For more histogram key word arguments, please see https://seaborn.pydata.org/generated/seaborn.distplot.html

        Parameters
        ----------
        x: str or str(s)
            Column(s) to plot histograms for.

        bins : argument for matplotlib hist(), or None, optional
            Specification of hist bins, or None to use Freedman-Diaconis rule.

        hist : bool, optional
            Whether to plot a (normed) histogram.

        kde : bool, optional
            Whether to plot a gaussian kernel density estimate.

        rug : bool, optional
            Whether to draw a rugplot on the support axis.

        fit : random variable object, optional
            An object with fit method, returning a tuple that can be passed to a pdf method a positional arguments following an grid of values to evaluate the pdf on.

        output_file : str, optional
            Output file name for image with extension (i.e. jpeg, png, etc.)

        Examples
        --------
        >>> data.histogram('col1')
        >>> data.histogram('col1', 'col2')
        >>> data.histogram('col1', kde=False)
        >>> data.histogram('col1', 'col2', hist=False)
        >>> data.histogram('col1', kde=False, fit=stat.normal)
        >>> data.histogram('col1', kde=False, output_file='hist.png')
        """

        histogram(list(x), data=self.data, output_file=output_file, **kwargs)

        if output_file and self.report:  # pragma: no cover
            self.report.write_image(output_file)



import unittest

import numpy as np
import pandas as pd
import shutil
from pathlib import Path
from aethos import Data


class TestFeatureExtraction(unittest.TestCase):
    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(str(Path.home()) + "/.aethos/reports/")

    def test_featureextractiontext_bow(self):

        list_of_sentences = ["Hi my name is pyml", "Hi name pyml"]

        columns = ["text"]
        data = pd.DataFrame(list_of_sentences, columns=columns)

        feature = Data(
            x_train=data, test_split_percentage=0.5, split=False, report_name="test"
        )
        feature.bag_of_words(keep_col=False)
        validate = feature.x_train.values.tolist()

        self.assertListEqual(validate, [[1, 1, 1, 1, 1], [1, 0, 0, 1, 1]])

    def test_featureextractiontext_bow_keepcol(self):

        list_of_sentences = ["Hi my name is pyml", "Hi name pyml"]

        columns = ["text"]
        data = pd.DataFrame(list_of_sentences, columns=columns)

        feature = Data(
            x_train=data, test_split_percentage=0.5, split=False, report_name="test"
        )
        feature.bag_of_words(keep_col=True)
        validate = feature.x_train.values.tolist()

        self.assertListEqual(
            validate,
            [["Hi my name is pyml", 1, 1, 1, 1, 1], ["Hi name pyml", 1, 0, 0, 1, 1]],
        )

    def test_featureextractiontext_tfidf(self):

        list_of_sentences = ["Hi my name is pyml", "Hi name pyml"]
        columns = ["text"]
        data = pd.DataFrame(list_of_sentences, columns=columns)

        feature = Data(
            x_train=data, test_split_percentage=0.5, split=False, report_name="test"
        )
        feature.tfidf(keep_col=False, lowercase=False, stop_words="english")
        validate = feature.x_train.shape[1]

        self.assertEqual(validate, 2)

    def test_featureextractiontext_splittfidf(self):

        list_of_sentences = ["Hi my name is pyml", "Hi name pyml"]
        columns = ["text"]
        data = pd.DataFrame(list_of_sentences, columns=columns)

        feature = Data(x_train=data, test_split_percentage=0.5, report_name="test")
        feature.tfidf("text", keep_col=False, lowercase=False, stop_words="english")
        validate = feature.x_train.shape[1]

        self.assertEqual(validate, 2)

    def test_featureextractiontext_tfidf_keepcol(self):

        list_of_sentences = ["Hi my name is pyml", "Hi name pyml"]
        columns = ["text"]
        data = pd.DataFrame(list_of_sentences, columns=columns)

        feature = Data(
            x_train=data, test_split_percentage=0.5, split=False, report_name="test"
        )
        feature.tfidf(keep_col=True, lowercase=False, stop_words="english")
        validate = feature.x_train.shape[1]

        self.assertEqual(validate, 3)

    def test_featureextractioncategorical_onehot(self):

        normal_data = [
            ["Green", "Green", "Car"],
            ["Green", "Other", "Truck"],
            ["Blue", "Other", "Truck"],
        ]

        columns = ["col1", "col2", "col3"]
        data = pd.DataFrame(normal_data, columns=columns)

        feature = Data(
            x_train=data, test_split_percentage=0.5, split=False, report_name="test"
        )
        feature.onehot_encode(list_of_cols=["col1", "col3"], keep_col=False)
        validate = feature.x_train.values.tolist()

        self.assertListEqual(
            validate,
            [["Green", 0, 1, 1, 0], ["Other", 0, 1, 0, 1], ["Other", 1, 0, 0, 1]],
        )

    def test_featureextractiontext_nltkpostag(self):

        normal_data = [
            "hi welcome to aethos.",
            "This application automates common Data Science/ML analysis tasks.",
        ]

        columns = ["text"]
        data = pd.DataFrame(normal_data, columns=columns)

        feature = Data(x_train=data, test_split_percentage=0.5, report_name="test")
        feature.postag_nltk()
        validate = feature.x_train.shape[1] == 2 and feature.x_test.shape[1] == 2

        self.assertTrue(validate, 2)

    def test_feature_fulldata_apply(self):

        data = [[1, 0, 1], [0, 2, 0], [1, 0, 1]]

        columns = ["col1", "col2", "col3"]
        data = pd.DataFrame(data, columns=columns)

        feature = Data(x_train=data, split=False, report_name="test")
        feature.apply(lambda x: x["col1"] > 0, "new_col")
        validate = "new_col" in feature.x_train.columns

        self.assertTrue(validate)

    def test_feature_splitdata_apply(self):

        data = [["py", 0, 1], ["auto", 2, 0], ["ml", 0, 1]]

        columns = ["col1", "col2", "col3"]
        data = pd.DataFrame(data, columns=columns)

        feature = Data(x_train=data, test_split_percentage=0.33, report_name="test")
        feature.apply(lambda x: x["col1"], "new_col")
        validate = (
            "new_col" in feature.x_train.columns and "new_col" in feature.x_test.columns
        )

        self.assertTrue(validate)

    def test_feature_labelencoder(self):

        data = [["canada", "green", 1], ["canada", "green", 1], ["canada", "green", 0]]

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        feature = Data(x_train=data, test_split_percentage=0.33, report_name="test")
        feature.encode_labels("col1", "col2")

        self.assertTrue(True)

    def test_feature_polynomial(self):

        data = np.arange(6).reshape(3, 2)

        data = pd.DataFrame(data=data, columns=["col1", "col2"])

        feature = Data(x_train=data, test_split_percentage=0.33, report_name="test")
        feature.polynomial_features()

        validate = feature.x_train.shape[1] == 6 and feature.x_test.shape[1] == 6

        self.assertTrue(validate)

    def test_featureextractiontext_spacypostag(self):

        normal_data = [
            "hi welcome to aethos.",
            "This application automates common Data Science/ML analysis tasks.",
        ]

        columns = ["text"]
        data = pd.DataFrame(normal_data, columns=columns)

        feature = Data(x_train=data, test_split_percentage=0.5, report_name="test")
        feature.postag_spacy()
        validate = feature.x_train.shape[1] == 2 and feature.x_test.shape[1] == 2

        self.assertTrue(validate, 2)

    def test_featureextractiontext_spacyphrases(self):

        normal_data = [
            "hi welcome to aethos.",
            "This application automates common Data Science/ML analysis tasks.",
        ]

        columns = ["text"]
        data = pd.DataFrame(normal_data, columns=columns)

        feature = Data(x_train=data, test_split_percentage=0.5, report_name="test")
        feature.nounphrases_spacy()
        validate = feature.x_train.shape[1] == 2 and feature.x_test.shape[1] == 2

        self.assertTrue(validate, 2)

    def test_featureextractiontext_nltkphrases(self):

        normal_data = [
            "hi welcome to aethos.",
            "This application automates common Data Science/ML analysis tasks.",
        ]

        columns = ["text"]
        data = pd.DataFrame(normal_data, columns=columns)

        feature = Data(x_train=data, test_split_percentage=0.5, report_name="test")
        feature.nounphrases_nltk()
        validate = feature.x_train.shape[1] == 2 and feature.x_test.shape[1] == 2

        self.assertTrue(validate, 2)

    def test_featureextractiontext_hash_keepcol(self):

        list_of_sentences = ["Hi my name is pyml", "Hi name pyml"]

        columns = ["text"]
        data = pd.DataFrame(list_of_sentences, columns=columns)

        feature = Data(x_train=data, test_split_percentage=0.5, report_name="test")
        feature.text_hash(keep_col=True, n_features=5)

        self.assertTrue(True)

    def test_feature_pca(self):

        data = np.arange(6).reshape(3, 2)

        data = pd.DataFrame(data=data, columns=["col1", "col2"])

        feature = Data(x_train=data, test_split_percentage=0.33, report_name="test")
        feature.pca(n_components=2)

        validate = feature.x_train.shape[1] == 2 and feature.x_test.shape[1] == 2

        self.assertTrue(validate)

    def test_feature_pcatarget(self):

        data = np.arange(9).reshape(3, 3)

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        feature = Data(
            x_train=data,
            test_split_percentage=0.33,
            target_field="col3",
            report_name="test",
        )
        feature.pca(n_components=2)

        validate = feature.col3 is not None and feature.x_train.shape[1] == 3

        self.assertTrue(validate)

    def test_util_corr(self):

        int_missing_data = [
            [1, 8, 3, 4],
            [2, 8, 5, 1],
            [3, 8, 5, 7],
            [4, 8, 5, 2],
            [5, 8, 5, 9],
            [6, 8, 5, 3],
        ]

        columns = ["col1", "col2", "col3", "col4"]
        data = pd.DataFrame(int_missing_data, columns=columns)

        feat = Data(x_train=data, test_split_percentage=0.5, report_name="test")
        feat.drop_correlated_features()

        validate = feat.x_test.columns.tolist() == feat.x_train.columns.tolist() and feat.x_test.columns.tolist() == [
            "col2",
            "col3",
            "col4",
        ]

        self.assertTrue(True)


if __name__ == "__main__":
    unittest.main()



"""
This file contains the following methods:

remove_columns_threshold
remove_rows_threshold
remove_duplicate_rows
remove_duplicate_columns
replace_missing_random_discrete
replace_missing_knn
replace_missing_interpolate
replace_missing_fill
replace_missing_indicator
remove_unique_columns_threshold
"""
import warnings

import numpy as np
import pandas as pd
from aethos.util import drop_replace_columns
from sklearn.impute import MissingIndicator, KNNImputer


def remove_columns_threshold(x_train, x_test=None, threshold=0.5):
    """
    Remove columns from the dataframe that have more than the threshold value of missing rows.
    Example: Remove columns where > 50% of the data is missing

    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Dataset

    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.

    threshold : float
        Threshold value between 0 and 1 that if the column
        has more than the specified threshold of missing values, it is removed. 
        by default, 0.5
    
    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific column are missing

    Returns 2 Dataframes if x_test is provided.
    """

    if threshold > 1 or threshold < 0:
        raise ValueError("Threshold cannot be greater than 1 or less than 0.")

    criteria_meeting_columns = x_train.columns[x_train.isnull().mean() < threshold]

    if x_test is not None:
        return x_train[criteria_meeting_columns], x_test[criteria_meeting_columns]
    else:
        return x_train[criteria_meeting_columns], None


def remove_rows_threshold(x_train: pd.DataFrame, x_test=None, threshold=0.5):
    """
    Remove rows from the dataframe that have more than the threshold value of missing rows.
    Example: Remove rows where > 50% of the data is missing
    
    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Training dataset, by default None.

    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.

    threshold : float
        Threshold value between 0 and 1 that if the row
        has more than the specified threshold of missing values, it is removed.
        by default, 0.5
    
    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific row are missing

    Returns 2 Dataframes if x_test is provided.
    """

    if threshold > 1 or threshold < 0:
        raise ValueError("Threshold cannot be greater than 1 or less than 0.")

    x_train = x_train.dropna(thresh=round(x_train.shape[1] * threshold), axis=0)

    if x_test is not None:
        x_test = x_test.dropna(thresh=round(x_test.shape[1] * threshold), axis=0)

    return x_train, x_test


def remove_duplicate_rows(x_train, x_test=None, list_of_cols=[]):
    """
    Removes rows that are exact duplicates of each other.
    
    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Dataset

    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.
    
    list_of_cols: list, optional
        Columns to check if their values are duplicated, by default []

    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific row are missing

    Returns 2 Dataframes if x_test is provided.
    """

    x_train = x_train.drop_duplicates(list_of_cols)

    if x_test is not None:
        x_test = x_test.drop_duplicates(list_of_cols)

    return x_train, x_test


def remove_duplicate_columns(x_train, x_test=None):
    """
    Removes columns whose values are exact duplicates of each other.
    
    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Dataset.

    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.
    
    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific row are missing

    Returns 2 Dataframes if x_test is provided.
    """

    x_train = x_train.T.drop_duplicates().T

    if x_test is not None:
        x_test = x_test.T.drop_duplicates().T

    return x_train, x_test


def replace_missing_random_discrete(x_train, x_test=None, list_of_cols=[]):
    """
    Replace missing values in with a random number based off the distribution (number of occurences) 
    of the data.

    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Dataset
        
    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.
        
    list_of_cols : list
        A list of specific columns to apply this technique to, by default []   
    
    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific row are missing

    Returns 2 Dataframes if x_test is provided.
    """

    for col in list_of_cols:
        probabilities = x_train[col].value_counts(normalize=True)

        missing_data = x_train[col].isnull()
        x_train.loc[missing_data, col] = np.random.choice(
            probabilities.index,
            size=len(x_train[missing_data]),
            replace=True,
            p=probabilities.values,
        )

        if x_test is not None:
            missing_data = x_test[col].isnull()
            x_test.loc[missing_data, col] = np.random.choice(
                probabilities.index,
                size=len(x_test[missing_data]),
                replace=True,
                p=probabilities.values,
            )

    return x_train, x_test


def replace_missing_knn(x_train, x_test=None, **knn_kwargs):
    """
    Replaces missing data using K nearest neighbors.
    
    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Dataset
        
    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.

    n_neighbors : int, optional
        Number of rows around the missing data to look at, by default 5
    
    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific row are missing

    Returns 2 Dataframes if x_test is provided.
    """

    neighbors = knn_kwargs.pop("n_neighbors", 5)
    columns = x_train.columns
    knn = KNNImputer(n_neighbors=neighbors, **knn_kwargs)

    train_knn_transformed = knn.fit_transform(x_train.values)

    if x_test is not None:
        warnings.warn(
            "If your test data does not come from the same distribution of the training data, it may lead to erroneous results."
        )
        test_knn_transformed = knn.fit_transform(x_test.values)

    return (
        pd.DataFrame(data=train_knn_transformed, columns=columns),
        pd.DataFrame(data=test_knn_transformed, columns=columns),
    )


def replace_missing_interpolate(x_train, x_test=None, list_of_cols=[], **inter_kwargs):
    """
    Replaces missing data using interpolation techniques.
    
    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Dataset
        
    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.
        
    list_of_cols : list
        A list of specific columns to apply this technique to, by default []
    
    method : str, optional
        Interpolation method, by default linear

    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific row are missing

    Returns 2 Dataframes if x_test is provided.
    """

    method = inter_kwargs.pop("method", "linear")

    for col in list_of_cols:
        x_train[col] = x_train[col].interpolate(method=method, **inter_kwargs)

        if x_test is not None:
            warnings.warn(
                "If test data does not come from the same distribution of the training data, it may lead to erroneous results."
            )
            x_test[col] = x_test[col].interpolate(method=method, **inter_kwargs)

    return x_train, x_test


def replace_missing_fill(
    x_train, x_test=None, list_of_cols=[], method="", **extra_kwargs
):
    """
    Replaces missing values with the known values ahead of it and behind it.
    
    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Dataset
        
    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.
        
    list_of_cols : list
        A list of specific columns to apply this technique to, by default []

    method : str
        Type of fill, by default ''
    
    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific row are missing

    Returns 2 Dataframes if x_test is provided.
    """

    # Handle erroneous input
    extra_kwargs.pop("method", method)

    for col in list_of_cols:
        x_train[col] = x_train[col].fillna(method=method, **extra_kwargs)

        if x_test is not None:
            x_test[col] = x_test[col].fillna(method=method, **extra_kwargs)

    return x_train, x_test


def replace_missing_indicator(
    x_train,
    x_test=None,
    list_of_cols=[],
    missing_indicator=1,
    valid_indicator=0,
    keep_col=False,
):
    """
    Adds a new column describing if the column provided is missing data
    
    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Dataset
        
    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.

    list_of_cols : list
        A list of specific columns to apply this technique to, by default []

    missing_indicator : int, optional
        Value to indicate missing data, by default 1

    valid_indicator : int, optional
        Value to indicate non missing data, by default 0

    keep_col : bool, optional
        True to keep column, False to replace it, by default False
    
    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific row are missing

    Returns 2 Dataframes if x_test is provided.
    """

    for col in list_of_cols:
        x_train[col + "_missing"] = [
            missing_indicator if x else valid_indicator for x in x_train[col].isnull()
        ]

        if not keep_col:
            x_train = x_train.drop([col], axis=1)

        if x_test is not None:
            x_test[col + "_missing"] = [
                missing_indicator if x else valid_indicator
                for x in x_test[col].isnull()
            ]

            if not keep_col:
                x_test = x_test.drop([col], axis=1)

    return x_train, x_test


def remove_constant_columns(x_train, x_test=None):
    """
    Removes columns that have only one unique value.

    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Dataset
        
    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.

    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific row are missing

    Returns 2 Dataframes if x_test is provided.
    """

    # If the number of unique values is not 0(all missing) or 1(constant or constant + missing)
    keep_columns = list(
        filter(lambda x: x_train.nunique()[x] not in [0, 1], x_train.columns)
    )

    x_train = x_train[keep_columns]

    if x_test is not None:
        x_test = x_test[keep_columns]

    return x_train, x_test


def remove_unique_columns(x_train, x_test):
    """
    Remove columns that have only unique values.

    Parameters
    ----------
    x_train: Dataframe or array like - 2d
        Dataset
        
    x_test: Dataframe or array like - 2d
        Testing dataset, by default None.

    Returns
    -------
    Dataframe, *Dataframe
        Transformed dataframe with rows with a missing values in a specific row are missing

    Returns 2 Dataframes if x_test is provided.
    """

    # If the number of unique values is not 0(all missing) or 1(constant or constant + missing)
    keep_columns = list(
        filter(lambda x: x_train.nunique()[x] != x_train.shape[0], x_train.columns)
    )

    x_train = x_train[keep_columns]

    if x_test is not None:
        x_test = x_test[keep_columns]

    return x_train, x_test



import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import scipy as sc
import swifter
from aethos.config import technique_reason_repo
from scipy.stats.stats import ks_2samp
from sklearn.ensemble import ExtraTreesClassifier
from sklearn.metrics import classification_report
from sklearn.model_selection import StratifiedKFold, cross_val_predict
from tqdm import tqdm


class Stats(object):
    def predict_data_sample(self):
        """
        Identifies how similar the train and test set distribution are by trying to predict whether each sample belongs
        to the train or test set using Random Forest, 10 Fold Stratified Cross Validation.

        The lower the F1 score, the more similar the distributions are as it's harder to predict which sample belongs to which distribution.

        Credit: https://www.kaggle.com/nanomathias/distribution-of-test-vs-training-data#1.-t-SNE-Distribution-Overview

        Returns
        -------
        Data:
            Returns a deep copy of the Data object.

        Examples
        --------
        >>> data.predict_data_sample()
        """

        if self.x_test is None or not self.target_field:
            raise ValueError(
                "Test data or target field must be set. They can be set by assigning values to the `target_field` or the `x_test` variable."
            )

        report_info = technique_reason_repo["stats"]["dist_compare"]["predict"]

        x_train = self.x_train.drop(self.target_field, axis=1)
        x_test = self.x_test.drop(self.target_field, axis=1)

        x_train["label"] = 1
        x_test["label"] = 0

        data = pd.concat([x_train, x_test], axis=0)
        label = data["label"].tolist()

        predictions = cross_val_predict(
            ExtraTreesClassifier(n_estimators=100),
            data.drop(columns=["label"]),
            label,
            cv=StratifiedKFold(n_splits=10, shuffle=True, random_state=42),
        )

        if self.report is not None:
            self.report.report_technique(report_info, [])

        print(classification_report(data["label"].tolist(), predictions))

        return self.copy()

    def ks_feature_distribution(self, threshold=0.1, show_plots=True):
        """
        Uses the Kolomogorov-Smirnov test see if the distribution in the training and test sets are similar.
        
        Credit: https://www.kaggle.com/nanomathias/distribution-of-test-vs-training-data#1.-t-SNE-Distribution-Overview

        Parameters
        ----------
        threshold : float, optional
            KS statistic threshold, by default 0.1

        show_plots : bool, optional
            True to show histograms of feature distributions, by default True

        Returns
        -------
        DataFrame
            Columns that are significantly different in the train and test set.

        Examples
        --------
        >>> data.ks_feature_distribution()
        >>> data.ks_feature_distribution(threshold=0.2)
        """

        if self.x_test is None:
            raise ValueError(
                "Data must be split into train and test set. Please set the `x_test` variable."
            )

        report_info = technique_reason_repo["stats"]["dist_compare"]["ks"]

        diff_data = []
        diff_df = None

        for col in tqdm(self.x_train.columns):
            statistic, pvalue = ks_2samp(
                self.x_train[col].values, self.x_test[col].values
            )

            if pvalue <= 0.05 and np.abs(statistic) > threshold:
                diff_data.append(
                    {
                        "feature": col,
                        "p": np.round(pvalue, 5),
                        "statistic": np.round(np.abs(statistic), 2),
                    }
                )

        if diff_data:
            diff_df = pd.DataFrame(diff_data).sort_values(
                by=["statistic"], ascending=False
            )

            if show_plots:
                n_cols = 4
                n_rows = int(len(diff_df) / n_cols) + 1

                _, ax = plt.subplots(n_rows, n_cols, figsize=(40, 6 * n_rows))

                for i, (_, row) in enumerate(diff_df.iterrows()):
                    if i >= len(ax):
                        break
                    extreme = np.max(
                        np.abs(
                            self.x_train[row.feature].tolist()
                            + self.x_test[row.feature].tolist()
                        )
                    )
                    self.x_train.loc[:, row.feature].swifter.apply(np.log1p).hist(
                        ax=ax[i],
                        alpha=0.6,
                        label="Train",
                        density=True,
                        bins=np.arange(-extreme, extreme, 0.25),
                    )

                    self.x_test.loc[:, row.feature].swifter.apply(np.log1p).hist(
                        ax=ax[i],
                        alpha=0.6,
                        label="Train",
                        density=True,
                        bins=np.arange(-extreme, extreme, 0.25),
                    )

                    ax[i].set_title(f"Statistic = {row.statistic}, p = {row.p}")
                    ax[i].set_xlabel(f"Log({row.feature})")
                    ax[i].legend()

                plt.tight_layout()
                plt.show()

            if self.report is not None:
                self.report.report_technique(report_info, [])

        return diff_df



import inspect
import multiprocessing as mp
import os
import pickle
import warnings
from functools import partial, wraps
from pathlib import Path

import matplotlib.pyplot as plt
from aethos.config import DEFAULT_MODEL_DIR, cfg
from aethos.util import _make_dir
from aethos.visualizations.util import _make_image_dir
from pathos.multiprocessing import ProcessingPool
from sklearn.model_selection import GridSearchCV, KFold, StratifiedKFold
from yellowbrick.model_selection import CVScores, LearningCurve


def add_to_queue(model_function):
    @wraps(model_function)
    def wrapper(self, *args, **kwargs):
        default_kwargs = get_default_args(model_function)

        kwargs["run"] = kwargs.get("run", default_kwargs["run"])
        kwargs["model_name"] = kwargs.get("model_name", default_kwargs["model_name"])
        cv = kwargs.get("cv", False)

        if not _validate_model_name(self, kwargs["model_name"]):
            raise AttributeError("Invalid model name. Please choose another one.")

        if kwargs["run"] or cv:
            return model_function(self, *args, **kwargs)
        else:
            kwargs["run"] = True
            self._queued_models[kwargs["model_name"]] = partial(
                model_function, self, *args, **kwargs
            )

    return wrapper


def get_default_args(func):
    """
    Gets default arguments from a function.
    """

    signature = inspect.signature(func)

    return {
        k: v.default
        for k, v in signature.parameters.items()
        if v.default is not inspect.Parameter.empty
    }


def run_gridsearch(model, gridsearch, cv=5, scoring="accuracy", **gridsearch_kwargs):
    """
    Runs Gridsearch on a model
    
    Parameters
    ----------
    model : Model
        Model to run gridsearch on

    gridsearch : dict
        Dict of params to test

    cv : int, Crossvalidation Generator, optional
        Cross validation method, by default 12
    
    scoring : str
        Scoring metric to use when evaluating models
    
    Returns
    -------
    Model
        Initialized Gridsearch model
    """

    if isinstance(gridsearch, dict):
        gridsearch_grid = gridsearch
        print(f"Gridsearching with the following parameters: {gridsearch_grid}")
    else:
        raise ValueError("Invalid Gridsearch input.")

    model = GridSearchCV(
        model, gridsearch_grid, cv=cv, scoring=scoring, **gridsearch_kwargs
    )

    return model


def run_crossvalidation(
    model, x_train, y_train, cv=5, scoring="accuracy", report=None, model_name=None
):
    """
    Runs cross validation on a certain model.
    
    Parameters
    ----------
    model : Model
        Model to cross validate

    x_train : nd-array
        Training data

    y_train : nd-array
        Testing data

    cv : int, Crossvalidation Generator, optional
        Cross validation method, by default 5

    scoring : str, optional
        Scoring method, by default 'accuracy'
    """

    fig, axes = plt.subplots(nrows=1, ncols=2, figsize=(15, 5))
    visualizer_scores = CVScores(model, cv=cv, scoring=scoring, ax=axes[0])
    visualizer_scores.fit(x_train, y_train)
    visualizer_scores.finalize()

    visualizer_lcurve = LearningCurve(model, cv=cv, scoring=scoring, ax=axes[1])
    visualizer_lcurve.fit(x_train, y_train)
    visualizer_lcurve.finalize()

    visualizer_scores.show()
    visualizer_lcurve.show()

    if report:  # pragma: no cover
        imgdir = _make_image_dir()
        fig.savefig(os.path.join(imgdir, model_name + ".svg"))


def _run_models_parallel(model_obj):
    """
    Runs queued models in parallel
    
    Parameters
    ----------
    model_obj : Model
        Model object
    """

    p = ProcessingPool(mp.cpu_count())

    results = p.map(_run, list(model_obj._queued_models.values()))

    for result in results:
        model_obj._models[result.model_name] = result

    p.close()
    p.join()


def _run(model):
    """
    Runs a model
        
    Returns
    -------
    Model
        Trained model
    """
    return model()


def _get_cv_type(cv_type, random_state, **kwargs):
    """
    Takes in cv type from the user and initiates the cross validation generator.
    
    Parameters
    ----------
    cv_type : int, str or None
        Crossvalidation type

    random_state : int
        Random seed
    
    Returns
    -------
    Cross Validation Generator
        CV Generator
    """

    if not cv_type:
        return None, kwargs

    if isinstance(cv_type, int):
        cv_type = cv_type
    elif cv_type == "kfold":
        n_splits = kwargs.pop("n_splits", 5)
        shuffle = kwargs.pop("shuffle", False)

        cv_type = KFold(n_splits=n_splits, shuffle=shuffle, random_state=random_state)
    elif cv_type == "strat-kfold":
        n_splits = kwargs.pop("n_splits", 5)
        shuffle = kwargs.pop("shuffle", False)

        cv_type = StratifiedKFold(
            n_splits=n_splits, shuffle=shuffle, random_state=random_state
        )
    else:
        raise ValueError("Cross Validation type is invalid.")

    return cv_type, kwargs


def to_pickle(model, name, project=False, project_name=None):
    """
    Writes model to a pickle file.
    
    Parameters
    ----------
    model: Model object
        Model object to serialize

    name : str
        Name of the model
        
    project : bool
        Whether to write to the project folder, by default False
    """

    if not project:
        if not cfg["models"]["dir"]:  # pragma: no cover
            path = DEFAULT_MODEL_DIR
        else:
            path = cfg["models"]["dir"]
    else:
        path = os.path.join(
            os.path.expanduser("~"), ".aethos", "projects", project_name, "app"
        )

    if not os.path.exists(path):
        os.makedirs(path)

    pickle.dump(model, open(os.path.join(path, name + ".pkl"), "wb"))


def _validate_model_name(model_obj, model_name: str) -> bool:
    """
    Validates the inputted model name. If the object already has an
    attribute with that model name, it is invalid
    
    Parameters
    ----------
    model_name : str
        Proposed name of the model
        
    model_obj : Model
        Model object
    
    Returns
    -------
    bool
        True if model name is valid, false otherwise
    """

    if hasattr(model_obj, model_name) and model_name not in model_obj._models:
        return False

    return True



from .model import Model
from .model_analysis import (
    ModelBase,
    ClassificationModel,
    RegressionModel,
    TextModel,
    UnsupervisedModel,
)

__all__ = [
    "Model",
    "ModelBase",
    "ClassificationModel",
    "RegressionModel",
    "TextModel",
    "UnsupervisedModel",
]



import os
import shutil
import unittest
from pathlib import Path

import numpy as np
import pandas as pd
from aethos import Model
from sklearn.datasets import make_blobs


class TestModelling(unittest.TestCase):
    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(str(Path.home()) + "/.aethos/reports/")
        shutil.rmtree(str(Path.home()) + "/.aethos/models/")
        shutil.rmtree(str(Path.home()) + "/.aethos/projects/")

    def test_text_gensim_summarize(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])

        model = Model(x_train=data, split=False)
        model.summarize_gensim("data", ratio=0.5, run=True)
        validate = model.data_summarized is not None

        self.assertTrue(validate)

    def test_text_view_gensim_summarize(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])

        model = Model(x_train=data, split=False)
        m = model.summarize_gensim("data", ratio=0.5, run=True)
        m.view("data", "data_summarized")

        self.assertTrue(True)

    def test_text_gensim_keywords(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])

        model = Model(x_train=data, split=False)
        model.extract_keywords_gensim("data", ratio=0.5, run=True)
        validate = model.data_extracted_keywords is not None

        self.assertTrue(validate)

    def test_text_gensim_w2v(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])

        model = Model(x_train=data, split=False)
        model.Word2Vec("data", prep=True, run=True, min_count=1)
        validate = model.w2v is not None

        self.assertTrue(validate)

    def test_text_gensim_lda(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])
        data["prep"] = pd.Series([text.split() for text in text_data])

        model = Model(x_train=data, test_split_percentage=0.5)
        model.LDA("prep")
        validate = model.lda is not None

        self.assertTrue(validate)

    def test_text_gensim_prep_lda(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])

        model = Model(x_train=data, test_split_percentage=0.5)
        model.LDA("data", prep=True)
        validate = model.lda is not None

        self.assertTrue(validate)

    def test_text_view_topics(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])
        data["prep"] = pd.Series([text.split() for text in text_data])

        model = Model(x_train=data, test_split_percentage=0.5)
        l = model.LDA("prep")
        l.view_topics()
        l.view_topic(1)

        self.assertTrue(True)

    def test_text_model_perplexity(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])
        data["prep"] = pd.Series([text.split() for text in text_data])

        model = Model(x_train=data, test_split_percentage=0.5)
        l = model.LDA("prep")
        l.model_perplexity()

        self.assertTrue(True)

    def test_text_coherence_score(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])
        data["prep"] = pd.Series([text.split() for text in text_data])

        model = Model(x_train=data, test_split_percentage=0.5)
        l = model.LDA("prep")
        l.coherence_score("prep")

        self.assertTrue(True)

    def test_text_view_topics(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])
        data["prep"] = pd.Series([text.split() for text in text_data])

        model = Model(x_train=data, test_split_percentage=0.5)
        l = model.LDA("prep")
        l.view_topics()
        l.view_topic(1)

        self.assertTrue(True)

    def test_text_w2vprep(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])
        data["prep"] = pd.Series([text.split() for text in text_data])

        model = Model(x_train=data, split=False)
        model.Word2Vec("prep", run=True, min_count=1)
        validate = model.w2v is not None

        self.assertTrue(validate)

    def test_text_d2v(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])

        model = Model(x_train=data, split=False)
        model.Doc2Vec("data", prep=True, run=True, min_count=1)
        validate = model.d2v is not None

        self.assertTrue(validate)

    def test_text_d2vprep(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])
        data["prep"] = pd.Series([text.split() for text in text_data])

        model = Model(x_train=data, split=False)
        model.Doc2Vec("prep", run=True, min_count=1)
        validate = model.d2v is not None

        self.assertTrue(validate)

    def test_model_getattr(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])

        model = Model(x_train=data, split=False)
        model.extract_keywords_gensim("data", ratio=0.5, model_name="model1", run=True)
        validate = model.model1 is not None

        self.assertTrue(validate)

    def test_model_addtoqueue(self):

        text_data = [
            "Hi my name is aethos. Please split me.",
            "This function is going to split by sentence. Automation is great.",
        ]

        data = pd.DataFrame(data=text_data, columns=["data"])

        model = Model(x_train=data, split=False)
        model.extract_keywords_gensim("data", ratio=0.5, model_name="model1", run=False)
        model.summarize_gensim("data", ratio=0.5, run=False)
        validate = len(model._queued_models)

        self.assertEqual(validate, 2)

    def test_model_kmeans(self):

        data, _ = make_blobs(n_samples=1000, n_features=12, centers=8, random_state=42)
        data = pd.DataFrame(data=data)

        model = Model(x_train=data, split=False)
        model.KMeans(n_clusters=3, random_state=0, run=True)
        validate = model.kmeans_clusters is not None

        self.assertTrue(validate)

    def test_model_kmeans_nok(self):

        data, _ = make_blobs(n_samples=1000, n_features=12, centers=8, random_state=42)
        data = pd.DataFrame(data=data)

        model = Model(x_train=data, split=False)
        model.KMeans(random_state=0, run=True)
        validate = model.kmeans_clusters is not None

        self.assertTrue(validate)

    def test_model_kmeans_split(self):

        data = [[1, 2], [2, 2], [2, 3], [8, 7], [8, 8], [25, 80]]

        data = pd.DataFrame(data=data, columns=["col1", "col2"])

        model = Model(x_train=data)
        model.KMeans(n_clusters=3, random_state=0, run=True)
        validate = (
            model.x_train_results.kmeans_clusters is not None
            and model.x_test_results.kmeans_clusters is not None
        )

        self.assertTrue(validate)

    def test_model_dbscan(self):

        data = [[1, 2], [2, 2], [2, 3], [8, 7], [8, 8], [25, 80]]

        data = pd.DataFrame(data=data, columns=["col1", "col2"])

        model = Model(x_train=data, split=False)
        model.DBScan(eps=3, min_samples=2, run=True)
        validate = model.dbscan_clusters is not None

        self.assertTrue(validate)

    def test_model_cluster_filter(self):

        data = [[1, 2], [2, 2], [2, 3], [8, 7], [8, 8], [25, 80]]

        data = pd.DataFrame(data=data, columns=["col1", "col2"])

        model = Model(x_train=data, split=False)
        model = model.DBScan(eps=3, min_samples=2, run=True)
        filtered = model.filter_cluster(0)
        validate = all(filtered.dbscan_clusters == 0)

        self.assertTrue(validate)

    def test_model_unsupervised_defaultgridsearch(self):

        data, _ = make_blobs(n_samples=1000, n_features=12, centers=8, random_state=42)

        data = pd.DataFrame(data=data)

        model = Model(x_train=data, target_field=5, report_name="gridsearch_test")

        gridsearch_params = {"max_iter": [300, 200]}
        model.KMeans(gridsearch=gridsearch_params, cv=2, run=True)

        self.assertTrue(True)

    def test_model_defaultgridsearch(self):

        data = [
            [1, 2, 1],
            [2, 2, 1],
            [2, 3, 1],
            [8, 7, 0],
            [8, 8, 0],
            [25, 80, 0],
            [1, 2, 1],
            [3, 2, 0],
            [1, 2, 1],
        ]

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", report_name="gridsearch_test")

        gridsearch_params = {"C": [0.2, 1]}
        model.LogisticRegression(gridsearch=gridsearch_params, cv=2, run=True)

        self.assertTrue(True)

    def test_model_logisticregression(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        validate = (
            model.x_train_results.log_predictions is not None
            and model.x_test_results.log_predictions is not None
        )

        self.assertTrue(validate)

    def test_model_confusionmatrix(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", report_name="confusion_report")
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.confusion_matrix()

        self.assertTrue(True)

    def test_model_all_score_metrics(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", report_name="metric_report")
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.metrics()

        self.assertTrue(True)

    def test_model_report_classificationreport(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(
            x_train=data, target_field="col3", report_name="classification_report"
        )
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.classification_report()

        self.assertTrue(True)

    def test_model_report_modelweights(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", report_name="modelweights")
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.model_weights()

        self.assertTrue(True)

    def test_plot_roccurve(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(
            x_train=data,
            target_field="col3",
            test_split_percentage=0.5,
            report_name="modelweights",
        )
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.roc_curve()

        self.assertTrue(True)

    def test_decision_plot(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.5)
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.decision_plot()

        self.assertTrue(True)

    def test_decision_plot_all(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.5)
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.decision_plot(num_samples="all")

        self.assertTrue(True)

    def test_decision_plot_sameaxis(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.5)
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        r = model.log_reg.decision_plot(sample_no=1)
        model.log_reg.decision_plot(
            sample_no=2, feature_order=r.feature_idx, xlim=r.xlim
        )

        self.assertTrue(True)

    def test_decision_plot_misclassified(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.5)
        model.LogisticRegression(random_state=2, run=True)
        model.log_reg.decision_plot(0.75, highlight_misclassified=True)

        self.assertTrue(True)

    def test_force_plot(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.5)
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.force_plot()

        self.assertTrue(True)

    def test_force_plot_misclassified(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.6)
        model.LogisticRegression(random_state=2, run=True)
        model.log_reg.force_plot(misclassified=True)

        self.assertTrue(True)

    def test_get_misclassified(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.5)
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.shap_get_misclassified_index()

        self.assertTrue(True)

    def test_summaryplot(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.5)
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.summary_plot()

        self.assertTrue(True)

    def test_dependence_plot(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.5)
        model.LogisticRegression(random_state=2, penalty="l2", run=True)
        model.log_reg.dependence_plot("col1")

        self.assertTrue(True)

    def test_local_multiprocessing(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(
            x_train=data,
            target_field="col3",
            test_split_percentage=0.5,
            report_name="modelweights",
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l1", run=True
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l2", run=True
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l3", run=True
        )

        model.run_models()

        self.assertTrue(len(model._models) == 3 and len(model._queued_models) == 0)

    def test_local_seriesprocessing(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(
            x_train=data,
            target_field="col3",
            test_split_percentage=0.5,
            report_name="modelweights",
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l1", run=True
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l2", run=True
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l3", run=True
        )

        model.run_models(method="series")

        self.assertTrue(len(model._models) == 3 and len(model._queued_models) == 0)

    def test_interpretmodel_behaviour_all(self):

        train_data = np.random.random_sample(size=(1000, 2))
        label_data = np.random.randint(0, 2, size=(1000, 1))

        data = pd.DataFrame(data=train_data, columns=["col1", "col2"])
        data["col3"] = label_data

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.2)
        model.LogisticRegression(random_state=2, run=True)
        model.log_reg.interpret_model_behavior(show=False)

        self.assertTrue(True)

    def test_interpretmodel_behaviour_dependence(self):

        data = np.random.randint(0, 2, size=(1000, 3))
        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.4)
        model.LogisticRegression(random_state=2, run=True)
        model.log_reg.interpret_model_behavior(method="dependence", show=False)

        self.assertTrue(True)

    def test_interpretmodel_predictions_all(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.6)
        model.LogisticRegression(random_state=2, run=True)
        model.log_reg.interpret_model_predictions(show=False)

        self.assertTrue(True)

    def test_interpretmodel_predictions_lime(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.6)
        model.LogisticRegression(random_state=2, run=True)
        model.log_reg.interpret_model_predictions(method="lime", show=False)

        self.assertTrue(True)

    def test_interpretmodel_performance_all(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.6)
        model.LogisticRegression(random_state=2, run=True)
        model.log_reg.interpret_model_performance(show=False)

        self.assertTrue(True)

    def test_interpretmodel_performance_roc(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.6)
        model.LogisticRegression(random_state=2, run=True)
        model.log_reg.interpret_model_performance(method="ROC", show=False)

        self.assertTrue(True)

    def test_interpret_model(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.4)
        model.LogisticRegression(random_state=2, run=True)
        model.log_reg.interpret_model(show=False)

        self.assertTrue(True)

    def test_interpret_model_prerun(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.4)
        model.LogisticRegression(random_state=2, run=True)
        model.log_reg.interpret_model_performance(method="ROC", show=False)
        model.log_reg.interpret_model(show=False)

        self.assertTrue(True)

    def test_compareclsmodels(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(
            x_train=data,
            target_field="col3",
            test_split_percentage=0.5,
            report_name="modelweights",
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l1", run=True
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l2", run=True
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l3", run=True
        )

        model.run_models(method="series")
        model.compare_models()

        self.assertTrue(True)

    def test_compareregmodels(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(
            x_train=data,
            target_field="col3",
            test_split_percentage=0.5,
            report_name="modelweights",
        )
        model.LinearRegression(model_name="l1", run=True)
        model.LinearRegression(model_name="l2", run=True)
        model.LinearRegression(model_name="l3", run=True)

        model.run_models(method="series")
        model.compare_models()

        self.assertTrue(True)

    def test_cv(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])
        model = Model(x_train=data, target_field="col3", test_split_percentage=0.2)
        model.LogisticRegression(cv=2, random_state=2)

        self.assertTrue(True)

    def test_unsupervisedcv(self):

        data, _ = make_blobs(n_samples=1000, n_features=12, centers=8, random_state=42)

        data = pd.DataFrame(data=data)
        model = Model(x_train=data, target_field=6, test_split_percentage=0.2)
        model.KMeans(cv=2, random_state=2)

        self.assertTrue(True)

    def test_stratified_cv(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])
        model = Model(x_train=data, target_field="col3", test_split_percentage=0.2)
        cv_values = model.LogisticRegression(cv="strat-kfold", n_splits=10, run=True)

        self.assertTrue(True)

    def test_del_model(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])
        model = Model(x_train=data, target_field="col3", test_split_percentage=0.2)
        model.LogisticRegression(random_state=2, run=True)
        model.delete_model("log_reg")

        self.assertTrue(len(model._models) == 0)

    def test_model_ridgeclassifier(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.RidgeClassification(random_state=2, run=True)
        validate = model.ridge_cls is not None

        self.assertTrue(validate)

    def test_model_sgdclassifier(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.SGDClassification(random_state=2, run=True)
        validate = model.sgd_cls is not None

        self.assertTrue(validate)

    def test_model_adaclassifier(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.ADABoostClassification(random_state=2, run=True)
        validate = model.ada_cls is not None

        self.assertTrue(validate)

    def test_model_bagclassifier(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.BaggingClassification(random_state=2, run=True)
        validate = model.bag_cls is not None

        self.assertTrue(validate)

    def test_model_boostingclassifier(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.GradientBoostingClassification(random_state=2, run=True)
        validate = model.grad_cls is not None

        self.assertTrue(validate)

    def test_model_isoforest(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data)
        model.IsolationForest(random_state=2, run=True)
        validate = model.iso_forest is not None

        self.assertTrue(validate)

    def test_model_oneclasssvm(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data)
        model.OneClassSVM(run=True)
        validate = model.ocsvm is not None

        self.assertTrue(validate)

    def test_model_rfclassifier(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.RandomForestClassification(random_state=2, run=True)
        validate = model.rf_cls is not None

        self.assertTrue(validate)

    def test_model_view_rfclassifier(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.RandomForestClassification(random_state=2, run=True)
        validate = model.rf_cls.view_tree()

        self.assertTrue(True)

    def test_model_bernoulli(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.BernoulliClassification(run=True)
        validate = model.bern is not None

        self.assertTrue(validate)

    def test_model_gaussian(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.GaussianClassification(run=True)
        validate = model.gauss is not None

        self.assertTrue(validate)

    def test_model_multinomial(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.MultinomialClassification(run=True)

        validate = model.multi is not None

        self.assertTrue(validate)

    def test_model_dtclassifier(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.DecisionTreeClassification(random_state=2, run=True)
        validate = model.dt_cls is not None

        self.assertTrue(validate)

    def test_model_linearsvc(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.LinearSVC(random_state=2, run=True)
        validate = model.linsvc is not None

        self.assertTrue(validate)

    def test_model_svc(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.SVC(random_state=2, run=True)
        validate = model.svc_cls is not None

        self.assertTrue(validate)

    def test_model_bayesianridge(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.BayesianRidgeRegression(run=True)
        validate = model.bayridge_reg is not None

        self.assertTrue(validate)

    def test_model_elasticnet(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.ElasticnetRegression(random_state=2, run=True)
        validate = model.elastic is not None

        self.assertTrue(validate)

    def test_model_lasso(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.LassoRegression(random_state=2, run=True)
        validate = model.lasso is not None

        self.assertTrue(validate)

    def test_model_linreg(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.LinearRegression()
        validate = model.lin_reg is not None

        self.assertTrue(validate)

    def test_model_ridgeregression(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.RidgeRegression(random_state=2, run=True)
        validate = model.ridge_reg is not None

        self.assertTrue(validate)

    def test_model_sgdregression(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.SGDRegression(random_state=2, run=True)
        validate = model.sgd_reg is not None

        self.assertTrue(validate)

    def test_model_adaregression(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.ADABoostRegression(random_state=2, run=True)
        validate = model.ada_reg is not None

        self.assertTrue(validate)

    def test_model_bgregression(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.BaggingRegression(random_state=2, run=True)
        validate = model.bag_reg is not None

        self.assertTrue(validate)

    def test_model_gbregression(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.GradientBoostingRegression(random_state=2, run=True)
        validate = model.grad_reg is not None

        self.assertTrue(validate)

    def test_model_rfregression(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.RandomForestRegression(random_state=2, run=True)
        validate = model.rf_reg is not None

        self.assertTrue(validate)

    def test_model_dtregression(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.DecisionTreeRegression(random_state=2, run=True)
        validate = model.dt_reg is not None

        self.assertTrue(validate)

    def test_model_view_dtregression(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.DecisionTreeRegression(random_state=2, run=True)
        validate = model.dt_reg.view_tree()

        self.assertTrue(True)

    def test_model_linearsvr(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.LinearSVR(random_state=2, run=True)
        validate = model.linsvr is not None

        self.assertTrue(validate)

    def test_model_view_linearsvr(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.LinearSVR(random_state=2, run=True)

        self.assertRaises(NotImplementedError, model.linsvr.view_tree)

    def test_model_svr(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.SVR(run=True)
        validate = model.svr_reg is not None

        self.assertTrue(validate)

    def test_model_xgbc(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.XGBoostClassification(run=True)
        validate = model.xgb_cls is not None

        self.assertTrue(validate)

    def test_model_view_xgbc(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.XGBoostRegression(run=True)
        validate = model.xgb_reg.view_tree()

        self.assertTrue(True)

    def test_model_xgbr(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.XGBoostRegression(run=True)
        validate = model.xgb_reg is not None

        self.assertTrue(validate)

    def test_model_lgbc(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.LightGBMClassification(run=True)
        validate = model.lgbm_cls is not None

        self.assertTrue(validate)

    def test_model_lgbr(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.LightGBMRegression(run=True)
        validate = model.lgbm_reg is not None

        self.assertTrue(True)

    def test_model_view_lgbr(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.LightGBMRegression(run=True)
        model.lgbm_reg.view_tree()

        self.assertTrue(True)

    def test_model_cbc(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.CatBoostClassification(run=True)
        validate = model.cb_cls is not None

        self.assertTrue(validate)

    def test_model_cbr(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.CatBoostRegression(run=True)
        validate = model.cb_reg is not None

        self.assertTrue(True)

    def test_model_cbr_gridsearch(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.CatBoostRegression(cv="kfold", gridsearch={"learning_rate": [0.03, 0.1]})
        validate = model.cb_reg is not None

        self.assertTrue(True)

    def test_model_cbr_cv(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.CatBoostRegression(cv="kfold")
        validate = model.cb_reg is not None

        self.assertTrue(True)

    def test_model_view_cbr(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.CatBoostClassification(run=True)
        model.cb_cls.view_tree()

        self.assertTrue(True)

    def test_model_agglom(self):

        data = [[1, 2], [2, 2], [2, 3], [8, 7], [8, 8], [25, 80]]

        data = pd.DataFrame(data=data, columns=["col1", "col2"])

        model = Model(x_train=data, split=False)
        model.AgglomerativeClustering(n_clusters=2, run=True)
        validate = model.agglom is not None

        self.assertTrue(validate)

    def test_model_meanshift(self):

        data = [[1, 2], [2, 2], [2, 3], [8, 7], [8, 8], [25, 80]]

        data = pd.DataFrame(data=data, columns=["col1", "col2"])

        model = Model(x_train=data, split=False)
        model.MeanShift(run=True)
        validate = model.mshift is not None

        self.assertTrue(validate)

    def test_model_gaussianmixture(self):

        data = [[1, 2], [2, 2], [2, 3], [8, 7], [8, 8], [25, 80]]

        data = pd.DataFrame(data=data, columns=["col1", "col2"])

        model = Model(x_train=data, split=False)
        model.GaussianMixtureClustering(run=True)
        validate = model.gm_cluster is not None

        self.assertTrue(validate)

    def test_plot_clusters2d(self):

        data = [[1, 2], [2, 2], [2, 3], [8, 7], [8, 8], [25, 80]]

        data = pd.DataFrame(data=data, columns=["col1", "col2"])

        model = Model(x_train=data, split=False)
        model.KMeans(n_clusters=3, random_state=0, run=True)
        model.km.plot_clusters()

        self.assertTrue(True)

    def test_plot_clusters3d(self):

        data = [[1, 2, 3], [2, 2, 3], [2, 3, 4], [8, 7, 5], [8, 8, 5], [25, 80, 4]]

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, split=False)
        model.KMeans(n_clusters=3, random_state=0, run=True)
        model.km.plot_clusters(dim=3)

        self.assertTrue(True)

    def test_ytrain_split(self):

        data = [[1, 0, 0], [0, 2, 3], [0, 3, 4], [1, 2, 3]]
        columns = ["col1", "col2", "col3"]
        data = pd.DataFrame(data, columns=columns)

        base = Model(
            x_train=data,
            x_test=None,
            split=True,
            target_field="col3",
            report_name=None,
            test_split_percentage=0.5,
        )

        validate = (
            base.x_train_results[base.target_field].tolist() == base.y_train.tolist()
            and len(base.y_train) == 2
        )

        self.assertTrue(validate)

    def test_ytrain_nosplit(self):

        data = [[1, 0, 0], [0, 2, 3], [0, 3, 4], [1, 2, 3]]
        columns = ["col1", "col2", "col3"]
        data = pd.DataFrame(data, columns=columns)

        base = Model(
            x_train=data,
            x_test=None,
            split=False,
            target_field="col3",
            report_name=None,
            test_split_percentage=0.5,
        )

        validate = (
            base.x_train_results[base.target_field].tolist() == base.y_train.tolist()
            and len(base.y_train) == 4
        )

        self.assertTrue(validate)

    def test_ytrain_dne(self):

        data = [[1, 0, 0], [0, 2, 3], [0, 3, 4], [1, 2, 3]]
        columns = ["col1", "col2", "col3"]
        data = pd.DataFrame(data, columns=columns)

        base = Model(
            x_train=data,
            x_test=None,
            split=True,
            target_field="",
            report_name=None,
            test_split_percentage=0.5,
        )

        base.y_train = [1, 1]
        validate = base.x_train_results["label"].tolist() == [
            1,
            1,
        ] and base.y_train.tolist() == [1, 1]

        self.assertTrue(validate)

    def test_ytest_split(self):

        data = [[1, 0, 0], [0, 2, 3], [0, 3, 4], [1, 2, 3]]
        columns = ["col1", "col2", "col3"]
        data = pd.DataFrame(data, columns=columns)

        base = Model(
            x_train=data,
            x_test=None,
            split=True,
            target_field="col3",
            report_name=None,
            test_split_percentage=0.5,
        )

        validate = (
            base.x_test_results[base.target_field].tolist() == base.y_test.tolist()
            and len(base.y_test) == 2
        )

        self.assertTrue(validate)

    def test_ytest_dne(self):

        data = [[1, 0, 0], [0, 2, 3], [0, 3, 4], [1, 2, 3]]
        columns = ["col1", "col2", "col3"]
        data = pd.DataFrame(data, columns=columns)

        base = Model(
            x_train=data,
            x_test=None,
            split=True,
            target_field="",
            report_name=None,
            test_split_percentage=0.5,
        )

        base.y_test = [1, 1]

        validate = base.y_test.tolist() == [1, 1] and base.x_test_results[
            "label"
        ].tolist() == [1, 1]

        self.assertTrue(validate)

    def test_pickle_model(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.ADABoostRegression(random_state=2, run=True)

        model.ada_reg.to_pickle()

        validate = os.path.exists(str(Path.home()) + "/.aethos/models/ada_reg.pkl")

        self.assertTrue(validate)

    def test_pickle_model_analysis(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.ADABoostRegression(random_state=2, run=True)

        model.to_pickle("ada_reg")

        validate = os.path.exists(str(Path.home()) + "/.aethos/models/ada_reg.pkl")

        self.assertTrue(validate)

    def test_model_create_service(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        model.LogisticRegression(random_state=2, run=True)

        model.to_service("log_reg", "test")

        self.assertTrue(True)

    def test_model_analysis_create_service(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3")
        m = model.LogisticRegression(random_state=2)

        m.to_service("test1")

        self.assertTrue(True)

    def test_setattr_new(self):

        int_missing_data = [[1, 0, 0, 1], [0, 2, 3, 1], [0, 3, 4, 1], [1, 2, 3, 1]]
        columns = ["col1", "col2", "col3", "col4"]
        data = pd.DataFrame(int_missing_data, columns=columns)

        base = Model(
            x_train=data,
            x_test=None,
            split=True,
            target_field="",
            report_name="test",
            test_split_percentage=0.5,
        )
        base["col5"] = 4

        self.assertListEqual(base.col5.tolist(), [4, 4])

    def test_setattr_testset(self):

        int_missing_data = [[1, 0, 0, 1], [0, 2, 3, 1], [0, 3, 4, 1], [1, 2, 3, 1]]
        columns = ["col1", "col2", "col3", "col4"]
        data = pd.DataFrame(int_missing_data, columns=columns)

        base = Model(
            x_train=data,
            x_test=None,
            split=True,
            target_field="",
            report_name="test",
            test_split_percentage=0.25,
        )
        base["col5"] = [4]

        self.assertListEqual(base.x_test_results["col5"].tolist(), [4])

    def test_setattr_trainset(self):

        int_missing_data = [[1, 0, 0, 1], [0, 2, 3, 1], [0, 3, 4, 1], [1, 2, 3, 1]]
        columns = ["col1", "col2", "col3", "col4"]
        data = pd.DataFrame(int_missing_data, columns=columns)

        base = Model(
            x_train=data,
            x_test=None,
            split=True,
            target_field="",
            report_name="test",
            test_split_percentage=0.75,
        )
        base["col5"] = [4]

        self.assertListEqual(base["col5"].tolist(), [4])

    def test_setattr_bothset(self):

        int_missing_data = [[1, 0, 0, 1], [0, 2, 3, 1], [0, 3, 4, 1], [1, 2, 3, 1]]
        columns = ["col1", "col2", "col3", "col4"]
        data = pd.DataFrame(int_missing_data, columns=columns)

        base = Model(
            x_train=data,
            x_test=None,
            split=True,
            target_field="",
            report_name="test",
            test_split_percentage=0.75,
        )
        base["col5"] = ([4], [4, 4, 4])

        self.assertListEqual(base["col5"].tolist(), [4])
        self.assertListEqual(base.x_test_results["col5"].tolist(), [4, 4, 4])

    def test_setattr_old(self):

        int_missing_data = [[1, 0, 0], [0, 2, 3], [0, 3, 4], [1, 2, 3]]
        columns = ["col1", "col2", "col3"]
        data = pd.DataFrame(int_missing_data, columns=columns)

        base = Model(
            x_train=data,
            x_test=None,
            split=True,
            target_field="",
            report_name="test",
            test_split_percentage=0.5,
        )
        base.target_field = "col3"

        self.assertEqual("col3", base.target_field)

    def test_list_models_empty(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(
            x_train=data,
            target_field="col3",
            test_split_percentage=0.5,
            report_name="modelweights",
        )

        model.list_models()

        self.assertTrue(True)

    def test_list_models(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(
            x_train=data,
            target_field="col3",
            test_split_percentage=0.5,
            report_name="modelweights",
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l1", run=False
        )
        model.LogisticRegression(
            random_state=2, penalty="l2", model_name="l2", run=True
        )

        model.list_models()

        self.assertTrue(True)

    def test_incorrect_model_name(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(x_train=data, target_field="col3", test_split_percentage=0.5,)

        self.assertRaises(
            AttributeError,
            model.LogisticRegression,
            random_state=2,
            penalty="l2",
            model_name="x_train",
            run=False,
        )

    def test_model_debug(self):

        data = np.random.randint(0, 2, size=(1000, 3))

        data = pd.DataFrame(data=data, columns=["col1", "col2", "col3"])

        model = Model(
            x_train=data,
            target_field="col3",
            test_split_percentage=0.5,
            report_name="modelweights",
        )

        model.help_debug()

        self.assertTrue(True)


if __name__ == "__main__":
    unittest.main()



import unittest

import numpy as np
import pandas as pd
import seaborn as sns
from pathlib import Path
import shutil

from aethos.core import Data


class Test_TestBase(unittest.TestCase):
    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(str(Path.home()) + "/.aethos/reports/")

    def test_histogram_1(self):

        data = sns.load_dataset("iris")

        base = Data(
            x_train=data,
            x_test=None,
            split=True,
            target_field="species",
            report_name="test",
            test_split_percentage=0.5,
        )

        base.histogram("sepal_length")

        self.assertTrue(True)

    def test_histogram_multi(self):

        data = sns.load_dataset("iris")

        base = Data(
            x_train=data,
            x_test=None,
            split=True,
            target_field="species",
            report_name="test",
            test_split_percentage=0.5,
        )

        base.histogram("sepal_length", "sepal_width")

        self.assertTrue(True)

    def test_pairplot(self):

        data = sns.load_dataset("iris")

        base = Data(
            x_train=data,
            x_test=None,
            split=True,
            target_field="species",
            report_name="test",
            test_split_percentage=0.5,
        )

        base.pairplot()

        self.assertTrue(True)

    def test_jointplot(self):

        data = sns.load_dataset("iris")

        base = Data(
            x_train=data,
            x_test=None,
            split=True,
            target_field="species",
            report_name="test",
            test_split_percentage=0.5,
        )

        base.jointplot(x="sepal_width", y="sepal_length")

        self.assertTrue(True)

    def test_rainccloudplot(self):

        data = sns.load_dataset("iris")

        base = Data(
            x_train=data,
            x_test=None,
            split=True,
            target_field="species",
            report_name="test",
            test_split_percentage=0.5,
        )

        base.raincloud(x="sepal_width", y="sepal_length")

        self.assertTrue(True)

    def test_barplot(self):

        data = sns.load_dataset("iris")

        base = Data(
            x_train=data,
            x_test=None,
            split=True,
            target_field="species",
            report_name="test",
            test_split_percentage=0.5,
        )

        base.barplot(
            x="species", y=["sepal_length", "sepal_width"], method="mean", orient="h",
        )
        base.barplot(
            x="species", y=["sepal_length", "sepal_width"], method="mean", orient="v",
        )

        self.assertTrue(True)

    def test_correlation_plot(self):

        data = pd.DataFrame(np.random.rand(100, 10))

        base = Data(
            x_train=data,
            x_test=None,
            split=True,
            target_field="col3",
            report_name="test",
            test_split_percentage=0.5,
        )

        base.correlation_matrix(data_labels=True, hide_mirror=True)

        self.assertTrue(True)

    def test_lineplot(self):

        np.random.seed(42)
        df = pd.DataFrame(
            {
                "Google": np.random.randn(1000) + 0.2,
                "Apple": np.random.randn(1000) + 0.17,
                "date": pd.date_range("1/1/2000", periods=1000),
            }
        )

        clean = Data(x_train=df, split=False)
        clean.lineplot(x="date", y=["Google", "Apple"], show_figure=False)

        self.assertTrue(True)



import os

import click


@click.group()
def main():
    pass


@main.command()
def create():
    """
    Creates a Data Science folder structure and files.
    """
    os.system("cookiecutter https://github.com/drivendata/cookiecutter-data-science")


@main.command()
def enable_extensions():
    """
    Enables jupyter extensions such as qgrid.
    """
    os.system("jupyter nbextension enable --py --sys-prefix widgetsnbextension")
    os.system("jupyter nbextension enable --py --sys-prefix qgrid")


@main.command()
def install_corpora():
    """
    Installs the necessary corpora from spaCy and NLTK for NLP analysis.
    """
    os.system("python3 -m textblob.download_corpora")
    os.system("python3 -c 'import nltk; nltk.download(\"stopwords\")'")
    os.system("python3 -m spacy download en")



import os
import subprocess
from pathlib import Path

from aethos.templates.util import _create_project_dir
from jinja2 import Environment, PackageLoader


class TemplateGenerator(object):

    # Prepare environment and source data
    env = Environment(
        loader=PackageLoader("aethos", "templates"),
        trim_blocks=True,
        lstrip_blocks=True,
    )

    project_dir = os.path.join(os.path.expanduser("~"), ".aethos", "projects")

    @classmethod
    def generate_service(cls, name: str, filename: str):
        """
        Generates the necessary files to run your model as a service.

        Generates the app.py, Dockerfile and requirements.txt file.
        
        Parameters
        ----------
        name : str
            Project name

        filename : str
            Model file name
        """

        _create_project_dir(cls.project_dir, name=name)

        files = ["main.py", "Dockerfile", "requirements.txt"]

        for file in files:
            script = cls.env.get_template("files/" + file.replace(".py", "")).render(
                name=name, filename=filename, service=True,
            )

            if file.endswith(".py") or file.endswith(".txt"):
                with open(
                    os.path.join(cls.project_dir, name, "app", file),
                    "w",
                    encoding="utf8",
                ) as f:
                    f.write(script)
            else:
                with open(
                    os.path.join(cls.project_dir, name, file), "w", encoding="utf8"
                ) as f:
                    f.write(script)

        print(f"Deployment files can be found at {cls.project_dir}/{name}.")
